# Stubs for six.moves
#
# Note: Commented out items means they weren't implemented at the time.
# Uncomment them when the modules have been added to the typeshed.
import sys

from io import StringIO as cStringIO
from builtins import filter as filter
from itertools import filterfalse as filterfalse
from builtins import input as input
from sys import intern as intern
from builtins import map as map
from os import getcwd as getcwd
from os import getcwdb as getcwdb
from builtins import range as range
from functools import reduce as reduce
from shlex import quote as shlex_quote
from io import StringIO as StringIO
from collections import UserDict as UserDict
from collections import UserList as UserList
from collections import UserString as UserString
from builtins import range as xrange
from builtins import zip as zip
from itertools import zip_longest as zip_longest
from . import builtins
from . import configparser
# import copyreg as copyreg
# import dbm.gnu as dbm_gnu
from . import _dummy_thread
from . import http_cookiejar
from . import http_cookies
from . import html_entities
from . import html_parser
from . import http_client
from . import email_mime_multipart
from . import email_mime_nonmultipart
from . import email_mime_text
from . import email_mime_base
from . import BaseHTTPServer
from . import CGIHTTPServer
from . import SimpleHTTPServer
from . import cPickle
from . import queue
from . import reprlib
from . import socketserver
from . import _thread
from . import tkinter
from . import tkinter_dialog
from . import tkinter_filedialog
# import tkinter.scrolledtext as tkinter_scrolledtext
# import tkinter.simpledialog as tkinter_simpledialog
# import tkinter.tix as tkinter_tix
from . import tkinter_ttk
from . import tkinter_constants
# import tkinter.dnd as tkinter_dnd
# import tkinter.colorchooser as tkinter_colorchooser
from . import tkinter_commondialog
from . import tkinter_tkfiledialog
# import tkinter.font as tkinter_font
# import tkinter.messagebox as tkinter_messagebox
# import tkinter.simpledialog as tkinter_tksimpledialog
from . import urllib_parse
from . import urllib_error
from . import urllib
from . import urllib_robotparser
# import xmlrpc.client as xmlrpc_client
# import xmlrpc.server as xmlrpc_server

if sys.version_info >= (3, 4):
    from importlib import reload as reload_module
else:
    from imp import reload as reload_module



# Stubs for six.moves.urllib.request
from urllib2 import urlopen as urlopen
from urllib2 import install_opener as install_opener
from urllib2 import build_opener as build_opener
from urllib import pathname2url as pathname2url
from urllib import url2pathname as url2pathname
from urllib import getproxies as getproxies
from urllib2 import Request as Request
from urllib2 import OpenerDirector as OpenerDirector
from urllib2 import HTTPDefaultErrorHandler as HTTPDefaultErrorHandler
from urllib2 import HTTPRedirectHandler as HTTPRedirectHandler
from urllib2 import HTTPCookieProcessor as HTTPCookieProcessor
from urllib2 import ProxyHandler as ProxyHandler
from urllib2 import BaseHandler as BaseHandler
from urllib2 import HTTPPasswordMgr as HTTPPasswordMgr
from urllib2 import HTTPPasswordMgrWithDefaultRealm as HTTPPasswordMgrWithDefaultRealm
from urllib2 import AbstractBasicAuthHandler as AbstractBasicAuthHandler
from urllib2 import HTTPBasicAuthHandler as HTTPBasicAuthHandler
from urllib2 import ProxyBasicAuthHandler as ProxyBasicAuthHandler
from urllib2 import AbstractDigestAuthHandler as AbstractDigestAuthHandler
from urllib2 import HTTPDigestAuthHandler as HTTPDigestAuthHandler
from urllib2 import ProxyDigestAuthHandler as ProxyDigestAuthHandler
from urllib2 import HTTPHandler as HTTPHandler
from urllib2 import HTTPSHandler as HTTPSHandler
from urllib2 import FileHandler as FileHandler
from urllib2 import FTPHandler as FTPHandler
from urllib2 import CacheFTPHandler as CacheFTPHandler
from urllib2 import UnknownHandler as UnknownHandler
from urllib2 import HTTPErrorProcessor as HTTPErrorProcessor
from urllib import urlretrieve as urlretrieve
from urllib import urlcleanup as urlcleanup
from urllib import URLopener as URLopener
from urllib import FancyURLopener as FancyURLopener
from urllib import proxy_bypass as proxy_bypass




import ssl
from typing import Any, AnyStr, Dict, List, Union, Optional, Mapping, Callable, Sequence, Tuple
from urllib import addinfourl
from httplib import HTTPResponse

_string = Union[str, unicode]

class URLError(IOError):
    reason = ...  # type: Union[str, BaseException]

class HTTPError(URLError, addinfourl):
    code = ...  # type: int
    headers = ...  # type: Dict[str, str]

class Request(object):
    host = ...  # type: str
    port = ...  # type: str
    data = ...  # type: str
    headers = ...  # type: Dict[str, str]
    unverifiable = ...  # type: bool
    type = ...  # type: Optional[str]
    origin_req_host = ...
    unredirected_hdrs = ...

    def __init__(self, url: str, data: Optional[str] = ..., headers: Dict[str, str] = ...,
                 origin_req_host: Optional[str] = ..., unverifiable: bool = ...) -> None: ...
    def __getattr__(self, attr): ...
    def get_method(self) -> str: ...
    def add_data(self, data) -> None: ...
    def has_data(self) -> bool: ...
    def get_data(self) -> str: ...
    def get_full_url(self) -> str: ...
    def get_type(self): ...
    def get_host(self) -> str: ...
    def get_selector(self): ...
    def set_proxy(self, host, type) -> None: ...
    def has_proxy(self) -> bool: ...
    def get_origin_req_host(self) -> str: ...
    def is_unverifiable(self) -> bool: ...
    def add_header(self, key: str, val: str) -> None: ...
    def add_unredirected_header(self, key: str, val: str) -> None: ...
    def has_header(self, header_name: str) -> bool: ...
    def get_header(self, header_name: str, default: Optional[str] = ...) -> str: ...
    def header_items(self): ...

class OpenerDirector(object):
    def add_handler(self, handler: BaseHandler) -> None: ...
    def open(self, url: Union[Request, _string], data: Optional[_string] = ..., timeout: int = ...): ...
    def error(self, proto: _string, *args: Any): ...

def urlopen(url: Union[Request, _string], data: Optional[_string] = ..., timeout: int = ...,
            cafile: Optional[_string] = ..., capath: Optional[_string] = ..., cadefault: bool = ...,
            context: Optional[ssl.SSLContext] = ...): ...
def install_opener(opener: OpenerDirector) -> None: ...
def build_opener(*handlers: BaseHandler) -> OpenerDirector: ...

class BaseHandler:
    handler_order = ...  # type: int
    parent = ...  # type: OpenerDirector

    def add_parent(self, parent: OpenerDirector) -> None: ...
    def close(self) -> None: ...
    def __lt__(self, other: Any) -> bool: ...

class HTTPErrorProcessor(BaseHandler):
    def http_response(self, request, response): ...

class HTTPDefaultErrorHandler(BaseHandler):
    def http_error_default(self, req, fp, code, msg, hdrs): ...

class HTTPRedirectHandler(BaseHandler):
    max_repeats = ...  # type: int
    max_redirections = ...  # type: int
    def redirect_request(self, req, fp, code, msg, headers, newurl): ...
    def http_error_301(self, req, fp, code, msg, headers): ...
    def http_error_302(self, req, fp, code, msg, headers): ...
    def http_error_303(self, req, fp, code, msg, headers): ...
    def http_error_307(self, req, fp, code, msg, headers): ...
    inf_msg = ...  # type: str


class ProxyHandler(BaseHandler):
    def __init__(self, proxies=None): ...
    def proxy_open(self, req, proxy, type): ...

class HTTPPasswordMgr:
    def __init__(self) -> None: ...
    def add_password(self, realm: _string, uri: Union[_string, Sequence[_string]], user: _string, passwd: _string) -> None: ...
    def find_user_password(self, realm: _string, authuri: _string) -> Tuple[Any, Any]: ...
    def reduce_uri(self, uri: _string, default_port: bool = ...) -> Tuple[Any, Any]: ...
    def is_suburi(self, base: _string, test: _string) -> bool: ...

class HTTPPasswordMgrWithDefaultRealm(HTTPPasswordMgr): ...

class AbstractBasicAuthHandler:
    def __init__(self, password_mgr: Optional[HTTPPasswordMgr] = ...) -> None: ...
    def http_error_auth_reqed(self, authreq, host, req, headers): ...
    def retry_http_basic_auth(self, host, req, realm): ...

class HTTPBasicAuthHandler(AbstractBasicAuthHandler, BaseHandler):
    auth_header = ...  # type: str
    def http_error_401(self, req, fp, code, msg, headers): ...

class ProxyBasicAuthHandler(AbstractBasicAuthHandler, BaseHandler):
    auth_header = ...  # type: str
    def http_error_407(self, req, fp, code, msg, headers): ...

class AbstractDigestAuthHandler:
    def __init__(self, passwd: Optional[HTTPPasswordMgr] = ...) -> None: ...
    def reset_retry_count(self) -> None: ...
    def http_error_auth_reqed(self, auth_header: str, host: str, req: Request,
                              headers: Mapping[str, str]) -> None: ...
    def retry_http_digest_auth(self, req: Request, auth: str) -> Optional[HTTPResponse]: ...
    def get_cnonce(self, nonce: str) -> str: ...
    def get_authorization(self, req: Request, chal: Mapping[str, str]) -> str: ...
    def get_algorithm_impls(self, algorithm: str) -> Tuple[Callable[[str], str], Callable[[str, str], str]]: ...
    def get_entity_digest(self, data: Optional[bytes], chal: Mapping[str, str]) -> Optional[str]: ...

class HTTPDigestAuthHandler(BaseHandler, AbstractDigestAuthHandler):
    auth_header = ...  # type: str
    handler_order = ...  # type: int
    def http_error_401(self, req, fp, code, msg, headers): ...

class ProxyDigestAuthHandler(BaseHandler, AbstractDigestAuthHandler):
    auth_header = ...  # type: str
    handler_order = ...  # type: int
    def http_error_407(self, req, fp, code, msg, headers): ...

class AbstractHTTPHandler(BaseHandler):
    def __init__(self, debuglevel: int = ...) -> None: ...
    def do_request_(self, request): ...
    def do_open(self, http_class, req): ...

class HTTPHandler(AbstractHTTPHandler):
    def http_open(self, req): ...
    def http_request(self, request): ...

class HTTPSHandler(AbstractHTTPHandler):
    def __init__(self, debuglevel=0, context=None): ...
    def https_open(self, req): ...
    def https_request(self, request): ...

class HTTPCookieProcessor(BaseHandler):
    def __init__(self, cookiejar=None): ...
    def http_request(self, request): ...
    def http_response(self, request, response): ...

class UnknownHandler(BaseHandler):
    def unknown_open(self, req): ...

class FileHandler(BaseHandler):
    def file_open(self, req): ...
    def get_names(self): ...
    def open_local_file(self, req): ...

class FTPHandler(BaseHandler):
    def ftp_open(self, req): ...
    def connect_ftp(self, user, passwd, host, port, dirs, timeout): ...

class CacheFTPHandler(FTPHandler):
    def __init__(self) -> None: ...
    def setTimeout(self, t): ...
    def setMaxConns(self, m): ...
    def check_cache(self): ...
    def clear_cache(self): ...

def parse_http_list(s: AnyStr) -> List[AnyStr]: ...
def parse_keqv_list(l: List[AnyStr]) -> Dict[AnyStr, AnyStr]: ...



from typing import Optional, List, Any, IO

def split(s: Optional[str], comments: bool = ..., posix: bool = ...) -> List[str]: ...

class shlex:
    def __init__(self, instream: IO[Any] = ..., infile: IO[Any] = ..., posix: bool = ...) -> None: ...
    def get_token(self) -> Optional[str]: ...
    def push_token(self, _str: str) -> None: ...
    def read_token(self) -> str: ...
    def sourcehook(self, filename: str) -> None: ...
    def push_source(self, stream: IO[Any], filename: str = ...) -> None: ...
    def pop_source(self) -> IO[Any]: ...
    def error_leader(self, file: str = ..., line: int = ...) -> str: ...

    commenters = ...  # type: str
    wordchars = ...  # type: str
    whitespace = ...  # type: str
    escape = ...  # type: str
    quotes = ...  # type: str
    escapedquotes = ...  # type: str
    whitespace_split = ...  # type: bool
    infile = ...  # type: IO[Any]
    source = ...  # type: Optional[str]
    debug = ...  # type: int
    lineno = ...  # type: int
    token = ...  # type: Any
    eof = ...  # type: Optional[str]



from typing import Any, Optional

def load_endpoint_json(path): ...
def merge_endpoints(defaults, additions): ...
def load_regions(): ...
def get_regions(service_name, region_cls: Optional[Any] = ..., connection_cls: Optional[Any] = ...): ...

class RegionInfo:
    connection = ...  # type: Any
    name = ...  # type: Any
    endpoint = ...  # type: Any
    connection_cls = ...  # type: Any
    def __init__(self, connection: Optional[Any] = ..., name: Optional[Any] = ..., endpoint: Optional[Any] = ..., connection_cls: Optional[Any] = ...) -> None: ...
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...
    def connect(self, **kw_params): ...



# Stubs for filecmp (Python 2/3)
import sys
from typing import AnyStr, Callable, Dict, Generic, Iterable, List, Optional, Sequence, Tuple, Union, Text

DEFAULT_IGNORES = ...  # type: List[str]

def cmp(f1: Union[bytes, Text], f2: Union[bytes, Text], shallow: Union[int, bool] = ...) -> bool: ...
def cmpfiles(a: AnyStr, b: AnyStr, common: Iterable[AnyStr],
             shallow: Union[int, bool] = ...) -> Tuple[List[AnyStr], List[AnyStr], List[AnyStr]]: ...

class dircmp(Generic[AnyStr]):
    def __init__(self, a: AnyStr, b: AnyStr,
                 ignore: Optional[Sequence[AnyStr]] = ...,
                 hide: Optional[Sequence[AnyStr]] = ...) -> None: ...

    left = ...  # type: AnyStr
    right = ...  # type: AnyStr
    hide = ...  # type: Sequence[AnyStr]
    ignore = ...  # type: Sequence[AnyStr]

    # These properties are created at runtime by __getattr__
    subdirs = ...  # type: Dict[AnyStr, dircmp[AnyStr]]
    same_files = ...  # type: List[AnyStr]
    diff_files = ...  # type: List[AnyStr]
    funny_files = ...  # type: List[AnyStr]
    common_dirs = ...  # type: List[AnyStr]
    common_files = ...  # type: List[AnyStr]
    common_funny = ...  # type: List[AnyStr]
    common = ...  # type: List[AnyStr]
    left_only = ...  # type: List[AnyStr]
    right_only = ...  # type: List[AnyStr]
    left_list = ...  # type: List[AnyStr]
    right_list = ...  # type: List[AnyStr]

    def report(self) -> None: ...
    def report_partial_closure(self) -> None: ...
    def report_full_closure(self) -> None: ...

    methodmap = ...  # type: Dict[str, Callable[[], None]]
    def phase0(self) -> None: ...
    def phase1(self) -> None: ...
    def phase2(self) -> None: ...
    def phase3(self) -> None: ...
    def phase4(self) -> None: ...
    def phase4_closure(self) -> None: ...

if sys.version_info >= (3,):
    def clear_cache() -> None: ...



from google.protobuf.message import (
    Message,
)
from typing import (
    Optional,
    Text,
)


class DoubleValue(Message):
    value = ...  # type: float

    def __init__(self,
                 value: Optional[float] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> DoubleValue: ...


class FloatValue(Message):
    value = ...  # type: float

    def __init__(self,
                 value: Optional[float] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> FloatValue: ...


class Int64Value(Message):
    value = ...  # type: int

    def __init__(self,
                 value: Optional[int] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> Int64Value: ...


class UInt64Value(Message):
    value = ...  # type: int

    def __init__(self,
                 value: Optional[int] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> UInt64Value: ...


class Int32Value(Message):
    value = ...  # type: int

    def __init__(self,
                 value: Optional[int] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> Int32Value: ...


class UInt32Value(Message):
    value = ...  # type: int

    def __init__(self,
                 value: Optional[int] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> UInt32Value: ...


class BoolValue(Message):
    value = ...  # type: bool

    def __init__(self,
                 value: Optional[bool] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> BoolValue: ...


class StringValue(Message):
    value = ...  # type: Text

    def __init__(self,
                 value: Optional[Text] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> StringValue: ...


class BytesValue(Message):
    value = ...  # type: str

    def __init__(self,
                 value: Optional[str] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> BytesValue: ...



from typing import Any, Optional

class Deleted:
    key = ...  # type: Any
    version_id = ...  # type: Any
    delete_marker = ...  # type: Any
    delete_marker_version_id = ...  # type: Any
    def __init__(self, key: Optional[Any] = ..., version_id: Optional[Any] = ..., delete_marker: bool = ..., delete_marker_version_id: Optional[Any] = ...) -> None: ...
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...

class Error:
    key = ...  # type: Any
    version_id = ...  # type: Any
    code = ...  # type: Any
    message = ...  # type: Any
    def __init__(self, key: Optional[Any] = ..., version_id: Optional[Any] = ..., code: Optional[Any] = ..., message: Optional[Any] = ...) -> None: ...
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...

class MultiDeleteResult:
    bucket = ...  # type: Any
    deleted = ...  # type: Any
    errors = ...  # type: Any
    def __init__(self, bucket: Optional[Any] = ...) -> None: ...
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...



import sys

from enum import IntEnum

if sys.version_info >= (3, 5):
    class HTTPStatus(IntEnum):

        def __init__(self, *a) -> None: ...

        phrase = ...  # type: str
        description = ...  # type: str

        CONTINUE = ...  # type: HTTPStatus
        SWITCHING_PROTOCOLS = ...  # type: HTTPStatus
        PROCESSING = ...  # type: HTTPStatus
        OK = ...  # type: HTTPStatus
        CREATED = ...  # type: HTTPStatus
        ACCEPTED = ...  # type: HTTPStatus
        NON_AUTHORITATIVE_INFORMATION = ...  # type: HTTPStatus
        NO_CONTENT = ...  # type: HTTPStatus
        RESET_CONTENT = ...  # type: HTTPStatus
        PARTIAL_CONTENT = ...  # type: HTTPStatus
        MULTI_STATUS = ...  # type: HTTPStatus
        ALREADY_REPORTED = ...  # type: HTTPStatus
        IM_USED = ...  # type: HTTPStatus
        MULTIPLE_CHOICES = ...  # type: HTTPStatus
        MOVED_PERMANENTLY = ...  # type: HTTPStatus
        FOUND = ...  # type: HTTPStatus
        SEE_OTHER = ...  # type: HTTPStatus
        NOT_MODIFIED = ...  # type: HTTPStatus
        USE_PROXY = ...  # type: HTTPStatus
        TEMPORARY_REDIRECT = ...  # type: HTTPStatus
        PERMANENT_REDIRECT = ...  # type: HTTPStatus
        BAD_REQUEST = ...  # type: HTTPStatus
        UNAUTHORIZED = ...  # type: HTTPStatus
        PAYMENT_REQUIRED = ...  # type: HTTPStatus
        FORBIDDEN = ...  # type: HTTPStatus
        NOT_FOUND = ...  # type: HTTPStatus
        METHOD_NOT_ALLOWED = ...  # type: HTTPStatus
        NOT_ACCEPTABLE = ...  # type: HTTPStatus
        PROXY_AUTHENTICATION_REQUIRED = ...  # type: HTTPStatus
        REQUEST_TIMEOUT = ...  # type: HTTPStatus
        CONFLICT = ...  # type: HTTPStatus
        GONE = ...  # type: HTTPStatus
        LENGTH_REQUIRED = ...  # type: HTTPStatus
        PRECONDITION_FAILED = ...  # type: HTTPStatus
        REQUEST_ENTITY_TOO_LARGE = ...  # type: HTTPStatus
        REQUEST_URI_TOO_LONG = ...  # type: HTTPStatus
        UNSUPPORTED_MEDIA_TYPE = ...  # type: HTTPStatus
        REQUESTED_RANGE_NOT_SATISFIABLE = ...  # type: HTTPStatus
        EXPECTATION_FAILED = ...  # type: HTTPStatus
        UNPROCESSABLE_ENTITY = ...  # type: HTTPStatus
        LOCKED = ...  # type: HTTPStatus
        FAILED_DEPENDENCY = ...  # type: HTTPStatus
        UPGRADE_REQUIRED = ...  # type: HTTPStatus
        PRECONDITION_REQUIRED = ...  # type: HTTPStatus
        TOO_MANY_REQUESTS = ...  # type: HTTPStatus
        REQUEST_HEADER_FIELDS_TOO_LARGE = ...  # type: HTTPStatus
        INTERNAL_SERVER_ERROR = ...  # type: HTTPStatus
        NOT_IMPLEMENTED = ...  # type: HTTPStatus
        BAD_GATEWAY = ...  # type: HTTPStatus
        SERVICE_UNAVAILABLE = ...  # type: HTTPStatus
        GATEWAY_TIMEOUT = ...  # type: HTTPStatus
        HTTP_VERSION_NOT_SUPPORTED = ...  # type: HTTPStatus
        VARIANT_ALSO_NEGOTIATES = ...  # type: HTTPStatus
        INSUFFICIENT_STORAGE = ...  # type: HTTPStatus
        LOOP_DETECTED = ...  # type: HTTPStatus
        NOT_EXTENDED = ...  # type: HTTPStatus
        NETWORK_AUTHENTICATION_REQUIRED = ...  # type: HTTPStatus



import sys
import typing
from typing import ClassVar as ClassVar
from typing import ContextManager as ContextManager
from typing import Counter as Counter
from typing import DefaultDict as DefaultDict
from typing import Deque as Deque
from typing import NewType as NewType
from typing import NoReturn as NoReturn
from typing import overload as overload
from typing import Text as Text
from typing import Type as Type
from typing import TYPE_CHECKING as TYPE_CHECKING
from typing import TypeVar, Any

_TC = TypeVar('_TC', bound=Type[object])
class _SpecialForm:
    def __getitem__(self, typeargs: Any) -> Any: ...
def runtime(cls: _TC) -> _TC: ...
Protocol: _SpecialForm = ...

if sys.version_info >= (3, 3):
    from typing import ChainMap as ChainMap

if sys.version_info >= (3, 5):
    from typing import AsyncIterable as AsyncIterable
    from typing import AsyncIterator as AsyncIterator
    from typing import AsyncContextManager as AsyncContextManager
    from typing import Awaitable as Awaitable
    from typing import Coroutine as Coroutine

if sys.version_info >= (3, 6):
    from typing import AsyncGenerator as AsyncGenerator



# NB: os2emxpath.pyi, posixpath.pyi, ntpath.pyi, and macpath.pyi must remain consistent!
from typing import Any
from genericpath import *  # noqa: F403

curdir = ...  # type: Any
pardir = ...  # type: Any
extsep = ...  # type: Any
sep = ...  # type: Any
pathsep = ...  # type: Any
defpath = ...  # type: Any
altsep = ...  # type: Any
devnull = ...  # type: Any

def normcase(s): ...
def isabs(s): ...
def join(a, *p): ...
def split(p): ...
def splitext(p): ...
def splitdrive(p): ...
def basename(p): ...
def dirname(p): ...
def islink(path): ...
def lexists(path): ...
def samefile(f1, f2): ...
def sameopenfile(fp1, fp2): ...
def samestat(s1, s2): ...
def ismount(path): ...
def walk(top, func, arg): ...
def expanduser(path): ...
def expandvars(path): ...
def normpath(path): ...
def abspath(path): ...
def realpath(filename): ...

supports_unicode_filenames = ...  # type: Any

def relpath(path, start=...): ...



"""Stub file for the 'signal' module."""

import sys
from enum import IntEnum
from typing import Any, Callable, List, Tuple, Dict, Generic, Union, Optional, Iterable, Set
from types import FrameType

class ItimerError(IOError): ...

ITIMER_PROF = ...  # type: int
ITIMER_REAL = ...  # type: int
ITIMER_VIRTUAL = ...  # type: int

NSIG = ...  # type: int

if sys.version_info >= (3, 5):
    class Signals(IntEnum):
        SIGABRT = ...
        SIGALRM = ...
        SIGBUS = ...
        SIGCHLD = ...
        SIGCLD = ...
        SIGCONT = ...
        SIGFPE = ...
        SIGHUP = ...
        SIGILL = ...
        SIGINT = ...
        SIGIO = ...
        SIGIOT = ...
        SIGKILL = ...
        SIGPIPE = ...
        SIGPOLL = ...
        SIGPROF = ...
        SIGPWR = ...
        SIGQUIT = ...
        SIGRTMAX = ...
        SIGRTMIN = ...
        SIGSEGV = ...
        SIGSTOP = ...
        SIGSYS = ...
        SIGTERM = ...
        SIGTRAP = ...
        SIGTSTP = ...
        SIGTTIN = ...
        SIGTTOU = ...
        SIGURG = ...
        SIGUSR1 = ...
        SIGUSR2 = ...
        SIGVTALRM = ...
        SIGWINCH = ...
        SIGXCPU = ...
        SIGXFSZ = ...

    class Handlers(IntEnum):
        SIG_DFL = ...
        SIG_IGN = ...

    SIG_DFL = Handlers.SIG_DFL
    SIG_IGN = Handlers.SIG_IGN

    class Sigmasks(IntEnum):
        SIG_BLOCK = ...
        SIG_UNBLOCK = ...
        SIG_SETMASK = ...

    SIG_BLOCK = Sigmasks.SIG_BLOCK
    SIG_UNBLOCK = Sigmasks.SIG_UNBLOCK
    SIG_SETMASK = Sigmasks.SIG_SETMASK

    _SIG = Signals
    _SIGNUM = Union[int, Signals]
    _HANDLER = Union[Callable[[Signals, FrameType], None], int, Handlers, None]
else:
    SIG_DFL = ...  # type: int
    SIG_IGN = ...  # type: int

    SIG_BLOCK = ...  # type: int
    SIG_UNBLOCK = ...  # type: int
    SIG_SETMASK = ...  # type: int

    _SIG = int
    _SIGNUM = int
    _HANDLER = Union[Callable[[int, FrameType], None], int, None]

SIGABRT = ...  # type: _SIG
SIGALRM = ...  # type: _SIG
SIGBUS = ...  # type: _SIG
SIGCHLD = ...  # type: _SIG
SIGCLD = ...  # type: _SIG
SIGCONT = ...  # type: _SIG
SIGFPE = ...  # type: _SIG
SIGHUP = ...  # type: _SIG
SIGILL = ...  # type: _SIG
SIGINT = ...  # type: _SIG
SIGIO = ...  # type: _SIG
SIGIOT = ...  # type: _SIG
SIGKILL = ...  # type: _SIG
SIGPIPE = ...  # type: _SIG
SIGPOLL = ...  # type: _SIG
SIGPROF = ...  # type: _SIG
SIGPWR = ...  # type: _SIG
SIGQUIT = ...  # type: _SIG
SIGRTMAX = ...  # type: _SIG
SIGRTMIN = ...  # type: _SIG
SIGSEGV = ...  # type: _SIG
SIGSTOP = ...  # type: _SIG
SIGSYS = ...  # type: _SIG
SIGTERM = ...  # type: _SIG
SIGTRAP = ...  # type: _SIG
SIGTSTP = ...  # type: _SIG
SIGTTIN = ...  # type: _SIG
SIGTTOU = ...  # type: _SIG
SIGURG = ...  # type: _SIG
SIGUSR1 = ...  # type: _SIG
SIGUSR2 = ...  # type: _SIG
SIGVTALRM = ...  # type: _SIG
SIGWINCH = ...  # type: _SIG
SIGXCPU = ...  # type: _SIG
SIGXFSZ = ...  # type: _SIG

CTRL_C_EVENT = 0  # Windows
CTRL_BREAK_EVENT = 0  # Windows

class struct_siginfo(Tuple[int, int, int, int, int, int, int]):
    def __init__(self, sequence: Iterable[int]) -> None: ...
    @property
    def si_signo(self) -> int: ...
    @property
    def si_code(self) -> int: ...
    @property
    def si_errno(self) -> int: ...
    @property
    def si_pid(self) -> int: ...
    @property
    def si_uid(self) -> int: ...
    @property
    def si_status(self) -> int: ...
    @property
    def si_band(self) -> int: ...

def alarm(time: int) -> int: ...

def default_int_handler(signum: int, frame: FrameType) -> None:
    raise KeyboardInterrupt()

def getitimer(which: int) -> Tuple[float, float]: ...

def getsignal(signalnum: _SIGNUM) -> _HANDLER:
    raise ValueError()

def pause() -> None: ...

def pthread_kill(thread_id: int, signum: int) -> None:
    raise OSError()

def pthread_sigmask(how: int, mask: Iterable[int]) -> Set[_SIGNUM]:
    raise OSError()

def set_wakeup_fd(fd: int) -> int: ...

def setitimer(which: int, seconds: float, interval: float = ...) -> Tuple[float, float]: ...

def siginterrupt(signalnum: int, flag: bool) -> None:
    raise OSError()

def signal(signalnum: _SIGNUM, handler: _HANDLER) -> _HANDLER:
    raise OSError()

def sigpending() -> Any:
    raise OSError()

def sigtimedwait(sigset: Iterable[int], timeout: float) -> Optional[struct_siginfo]:
    raise OSError()
    raise ValueError()

def sigwait(sigset: Iterable[int]) -> _SIGNUM:
    raise OSError()

def sigwaitinfo(sigset: Iterable[int]) -> struct_siginfo:
    raise OSError()



from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer,
)
from google.protobuf.message import (
    Message,
)
import builtins
from typing import (
    Iterable,
    List,
    Mapping,
    MutableMapping,
    Optional,
    Text,
    Tuple,
    cast,
)


class ForeignEnumProto2(int):

    @classmethod
    def Name(cls, number: int) -> str: ...

    @classmethod
    def Value(cls, name: str) -> ForeignEnumProto2: ...

    @classmethod
    def keys(cls) -> List[str]: ...

    @classmethod
    def values(cls) -> List[ForeignEnumProto2]: ...

    @classmethod
    def items(cls) -> List[Tuple[str, ForeignEnumProto2]]: ...


FOREIGN_FOO: ForeignEnumProto2
FOREIGN_BAR: ForeignEnumProto2
FOREIGN_BAZ: ForeignEnumProto2


class TestAllTypesProto2(Message):

    class NestedEnum(int):

        @classmethod
        def Name(cls, number: int) -> str: ...

        @classmethod
        def Value(cls, name: str) -> TestAllTypesProto2.NestedEnum: ...

        @classmethod
        def keys(cls) -> List[str]: ...

        @classmethod
        def values(cls) -> List[TestAllTypesProto2.NestedEnum]: ...

        @classmethod
        def items(cls) -> List[Tuple[str, TestAllTypesProto2.NestedEnum]]: ...
    FOO: NestedEnum
    BAR: NestedEnum
    BAZ: NestedEnum
    NEG: NestedEnum

    class NestedMessage(Message):
        a = ...  # type: int

        @property
        def corecursive(self) -> TestAllTypesProto2: ...

        def __init__(self,
                     a: Optional[int] = ...,
                     corecursive: Optional[TestAllTypesProto2] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(cls, s: str) -> TestAllTypesProto2.NestedMessage: ...

    class MapInt32Int32Entry(Message):
        key = ...  # type: int
        value = ...  # type: int

        def __init__(self,
                     key: Optional[int] = ...,
                     value: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapInt32Int32Entry: ...

    class MapInt64Int64Entry(Message):
        key = ...  # type: int
        value = ...  # type: int

        def __init__(self,
                     key: Optional[int] = ...,
                     value: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapInt64Int64Entry: ...

    class MapUint32Uint32Entry(Message):
        key = ...  # type: int
        value = ...  # type: int

        def __init__(self,
                     key: Optional[int] = ...,
                     value: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapUint32Uint32Entry: ...

    class MapUint64Uint64Entry(Message):
        key = ...  # type: int
        value = ...  # type: int

        def __init__(self,
                     key: Optional[int] = ...,
                     value: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapUint64Uint64Entry: ...

    class MapSint32Sint32Entry(Message):
        key = ...  # type: int
        value = ...  # type: int

        def __init__(self,
                     key: Optional[int] = ...,
                     value: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapSint32Sint32Entry: ...

    class MapSint64Sint64Entry(Message):
        key = ...  # type: int
        value = ...  # type: int

        def __init__(self,
                     key: Optional[int] = ...,
                     value: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapSint64Sint64Entry: ...

    class MapFixed32Fixed32Entry(Message):
        key = ...  # type: int
        value = ...  # type: int

        def __init__(self,
                     key: Optional[int] = ...,
                     value: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapFixed32Fixed32Entry: ...

    class MapFixed64Fixed64Entry(Message):
        key = ...  # type: int
        value = ...  # type: int

        def __init__(self,
                     key: Optional[int] = ...,
                     value: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapFixed64Fixed64Entry: ...

    class MapSfixed32Sfixed32Entry(Message):
        key = ...  # type: int
        value = ...  # type: int

        def __init__(self,
                     key: Optional[int] = ...,
                     value: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapSfixed32Sfixed32Entry: ...

    class MapSfixed64Sfixed64Entry(Message):
        key = ...  # type: int
        value = ...  # type: int

        def __init__(self,
                     key: Optional[int] = ...,
                     value: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapSfixed64Sfixed64Entry: ...

    class MapInt32FloatEntry(Message):
        key = ...  # type: int
        value = ...  # type: float

        def __init__(self,
                     key: Optional[int] = ...,
                     value: Optional[float] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapInt32FloatEntry: ...

    class MapInt32DoubleEntry(Message):
        key = ...  # type: int
        value = ...  # type: float

        def __init__(self,
                     key: Optional[int] = ...,
                     value: Optional[float] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapInt32DoubleEntry: ...

    class MapBoolBoolEntry(Message):
        key = ...  # type: bool
        value = ...  # type: bool

        def __init__(self,
                     key: Optional[bool] = ...,
                     value: Optional[bool] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(cls, s: str) -> TestAllTypesProto2.MapBoolBoolEntry: ...

    class MapStringStringEntry(Message):
        key = ...  # type: Text
        value = ...  # type: Text

        def __init__(self,
                     key: Optional[Text] = ...,
                     value: Optional[Text] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapStringStringEntry: ...

    class MapStringBytesEntry(Message):
        key = ...  # type: Text
        value = ...  # type: str

        def __init__(self,
                     key: Optional[Text] = ...,
                     value: Optional[str] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapStringBytesEntry: ...

    class MapStringNestedMessageEntry(Message):
        key = ...  # type: Text

        @property
        def value(self) -> TestAllTypesProto2.NestedMessage: ...

        def __init__(self,
                     key: Optional[Text] = ...,
                     value: Optional[TestAllTypesProto2.NestedMessage] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapStringNestedMessageEntry: ...

    class MapStringForeignMessageEntry(Message):
        key = ...  # type: Text

        @property
        def value(self) -> ForeignMessageProto2: ...

        def __init__(self,
                     key: Optional[Text] = ...,
                     value: Optional[ForeignMessageProto2] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapStringForeignMessageEntry: ...

    class MapStringNestedEnumEntry(Message):
        key = ...  # type: Text
        value = ...  # type: TestAllTypesProto2.NestedEnum

        def __init__(self,
                     key: Optional[Text] = ...,
                     value: Optional[TestAllTypesProto2.NestedEnum] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapStringNestedEnumEntry: ...

    class MapStringForeignEnumEntry(Message):
        key = ...  # type: Text
        value = ...  # type: ForeignEnumProto2

        def __init__(self,
                     key: Optional[Text] = ...,
                     value: Optional[ForeignEnumProto2] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MapStringForeignEnumEntry: ...

    class Data(Message):
        group_int32 = ...  # type: int
        group_uint32 = ...  # type: int

        def __init__(self,
                     group_int32: Optional[int] = ...,
                     group_uint32: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(cls, s: str) -> TestAllTypesProto2.Data: ...

    class MessageSetCorrect(Message):

        def __init__(self,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MessageSetCorrect: ...

    class MessageSetCorrectExtension1(Message):
        str = ...  # type: Text

        def __init__(self,
                     str: Optional[Text] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: builtins.str) -> TestAllTypesProto2.MessageSetCorrectExtension1: ...

    class MessageSetCorrectExtension2(Message):
        i = ...  # type: int

        def __init__(self,
                     i: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: str) -> TestAllTypesProto2.MessageSetCorrectExtension2: ...
    optional_int32 = ...  # type: int
    optional_int64 = ...  # type: int
    optional_uint32 = ...  # type: int
    optional_uint64 = ...  # type: int
    optional_sint32 = ...  # type: int
    optional_sint64 = ...  # type: int
    optional_fixed32 = ...  # type: int
    optional_fixed64 = ...  # type: int
    optional_sfixed32 = ...  # type: int
    optional_sfixed64 = ...  # type: int
    optional_float = ...  # type: float
    optional_double = ...  # type: float
    optional_bool = ...  # type: bool
    optional_string = ...  # type: Text
    optional_bytes = ...  # type: str
    optional_nested_enum = ...  # type: TestAllTypesProto2.NestedEnum
    optional_foreign_enum = ...  # type: ForeignEnumProto2
    optional_string_piece = ...  # type: Text
    optional_cord = ...  # type: Text
    repeated_int32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_int64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_uint32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_uint64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_sint32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_sint64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_fixed32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_fixed64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_sfixed32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_sfixed64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_float = ...  # type: RepeatedScalarFieldContainer[float]
    repeated_double = ...  # type: RepeatedScalarFieldContainer[float]
    repeated_bool = ...  # type: RepeatedScalarFieldContainer[bool]
    repeated_string = ...  # type: RepeatedScalarFieldContainer[Text]
    repeated_bytes = ...  # type: RepeatedScalarFieldContainer[str]
    repeated_nested_enum = ...  # type: RepeatedScalarFieldContainer[TestAllTypesProto2.NestedEnum]
    repeated_foreign_enum = ...  # type: RepeatedScalarFieldContainer[ForeignEnumProto2]
    repeated_string_piece = ...  # type: RepeatedScalarFieldContainer[Text]
    repeated_cord = ...  # type: RepeatedScalarFieldContainer[Text]
    oneof_uint32 = ...  # type: int
    oneof_string = ...  # type: Text
    oneof_bytes = ...  # type: str
    oneof_bool = ...  # type: bool
    oneof_uint64 = ...  # type: int
    oneof_float = ...  # type: float
    oneof_double = ...  # type: float
    oneof_enum = ...  # type: TestAllTypesProto2.NestedEnum
    fieldname1 = ...  # type: int
    field_name2 = ...  # type: int
    _field_name3 = ...  # type: int
    field__name4_ = ...  # type: int
    field0name5 = ...  # type: int
    field_0_name6 = ...  # type: int
    fieldName7 = ...  # type: int
    FieldName8 = ...  # type: int
    field_Name9 = ...  # type: int
    Field_Name10 = ...  # type: int
    FIELD_NAME11 = ...  # type: int
    FIELD_name12 = ...  # type: int
    __field_name13 = ...  # type: int
    __Field_name14 = ...  # type: int
    field__name15 = ...  # type: int
    field__Name16 = ...  # type: int
    field_name17__ = ...  # type: int
    Field_name18__ = ...  # type: int

    @property
    def optional_nested_message(self) -> TestAllTypesProto2.NestedMessage: ...

    @property
    def optional_foreign_message(self) -> ForeignMessageProto2: ...

    @property
    def recursive_message(self) -> TestAllTypesProto2: ...

    @property
    def repeated_nested_message(
        self) -> RepeatedCompositeFieldContainer[TestAllTypesProto2.NestedMessage]: ...

    @property
    def repeated_foreign_message(
        self) -> RepeatedCompositeFieldContainer[ForeignMessageProto2]: ...

    @property
    def map_int32_int32(self) -> MutableMapping[int, int]: ...

    @property
    def map_int64_int64(self) -> MutableMapping[int, int]: ...

    @property
    def map_uint32_uint32(self) -> MutableMapping[int, int]: ...

    @property
    def map_uint64_uint64(self) -> MutableMapping[int, int]: ...

    @property
    def map_sint32_sint32(self) -> MutableMapping[int, int]: ...

    @property
    def map_sint64_sint64(self) -> MutableMapping[int, int]: ...

    @property
    def map_fixed32_fixed32(self) -> MutableMapping[int, int]: ...

    @property
    def map_fixed64_fixed64(self) -> MutableMapping[int, int]: ...

    @property
    def map_sfixed32_sfixed32(self) -> MutableMapping[int, int]: ...

    @property
    def map_sfixed64_sfixed64(self) -> MutableMapping[int, int]: ...

    @property
    def map_int32_float(self) -> MutableMapping[int, float]: ...

    @property
    def map_int32_double(self) -> MutableMapping[int, float]: ...

    @property
    def map_bool_bool(self) -> MutableMapping[bool, bool]: ...

    @property
    def map_string_string(self) -> MutableMapping[Text, Text]: ...

    @property
    def map_string_bytes(self) -> MutableMapping[Text, str]: ...

    @property
    def map_string_nested_message(
        self) -> MutableMapping[Text, TestAllTypesProto2.NestedMessage]: ...

    @property
    def map_string_foreign_message(
        self) -> MutableMapping[Text, ForeignMessageProto2]: ...

    @property
    def map_string_nested_enum(
        self) -> MutableMapping[Text, TestAllTypesProto2.NestedEnum]: ...

    @property
    def map_string_foreign_enum(
        self) -> MutableMapping[Text, ForeignEnumProto2]: ...

    @property
    def oneof_nested_message(self) -> TestAllTypesProto2.NestedMessage: ...

    @property
    def data(self) -> TestAllTypesProto2.Data: ...

    def __init__(self,
                 optional_int32: Optional[int] = ...,
                 optional_int64: Optional[int] = ...,
                 optional_uint32: Optional[int] = ...,
                 optional_uint64: Optional[int] = ...,
                 optional_sint32: Optional[int] = ...,
                 optional_sint64: Optional[int] = ...,
                 optional_fixed32: Optional[int] = ...,
                 optional_fixed64: Optional[int] = ...,
                 optional_sfixed32: Optional[int] = ...,
                 optional_sfixed64: Optional[int] = ...,
                 optional_float: Optional[float] = ...,
                 optional_double: Optional[float] = ...,
                 optional_bool: Optional[bool] = ...,
                 optional_string: Optional[Text] = ...,
                 optional_bytes: Optional[str] = ...,
                 optional_nested_message: Optional[TestAllTypesProto2.NestedMessage] = ...,
                 optional_foreign_message: Optional[ForeignMessageProto2] = ...,
                 optional_nested_enum: Optional[TestAllTypesProto2.NestedEnum] = ...,
                 optional_foreign_enum: Optional[ForeignEnumProto2] = ...,
                 optional_string_piece: Optional[Text] = ...,
                 optional_cord: Optional[Text] = ...,
                 recursive_message: Optional[TestAllTypesProto2] = ...,
                 repeated_int32: Optional[Iterable[int]] = ...,
                 repeated_int64: Optional[Iterable[int]] = ...,
                 repeated_uint32: Optional[Iterable[int]] = ...,
                 repeated_uint64: Optional[Iterable[int]] = ...,
                 repeated_sint32: Optional[Iterable[int]] = ...,
                 repeated_sint64: Optional[Iterable[int]] = ...,
                 repeated_fixed32: Optional[Iterable[int]] = ...,
                 repeated_fixed64: Optional[Iterable[int]] = ...,
                 repeated_sfixed32: Optional[Iterable[int]] = ...,
                 repeated_sfixed64: Optional[Iterable[int]] = ...,
                 repeated_float: Optional[Iterable[float]] = ...,
                 repeated_double: Optional[Iterable[float]] = ...,
                 repeated_bool: Optional[Iterable[bool]] = ...,
                 repeated_string: Optional[Iterable[Text]] = ...,
                 repeated_bytes: Optional[Iterable[str]] = ...,
                 repeated_nested_message: Optional[Iterable[TestAllTypesProto2.NestedMessage]] = ...,
                 repeated_foreign_message: Optional[Iterable[ForeignMessageProto2]] = ...,
                 repeated_nested_enum: Optional[Iterable[TestAllTypesProto2.NestedEnum]] = ...,
                 repeated_foreign_enum: Optional[Iterable[ForeignEnumProto2]] = ...,
                 repeated_string_piece: Optional[Iterable[Text]] = ...,
                 repeated_cord: Optional[Iterable[Text]] = ...,
                 map_int32_int32: Optional[Mapping[int, int]]=...,
                 map_int64_int64: Optional[Mapping[int, int]]=...,
                 map_uint32_uint32: Optional[Mapping[int, int]]=...,
                 map_uint64_uint64: Optional[Mapping[int, int]]=...,
                 map_sint32_sint32: Optional[Mapping[int, int]]=...,
                 map_sint64_sint64: Optional[Mapping[int, int]]=...,
                 map_fixed32_fixed32: Optional[Mapping[int, int]]=...,
                 map_fixed64_fixed64: Optional[Mapping[int, int]]=...,
                 map_sfixed32_sfixed32: Optional[Mapping[int, int]]=...,
                 map_sfixed64_sfixed64: Optional[Mapping[int, int]]=...,
                 map_int32_float: Optional[Mapping[int, float]]=...,
                 map_int32_double: Optional[Mapping[int, float]]=...,
                 map_bool_bool: Optional[Mapping[bool, bool]]=...,
                 map_string_string: Optional[Mapping[Text, Text]]=...,
                 map_string_bytes: Optional[Mapping[Text, str]]=...,
                 map_string_nested_message: Optional[Mapping[Text, TestAllTypesProto2.NestedMessage]]=...,
                 map_string_foreign_message: Optional[Mapping[Text, ForeignMessageProto2]]=...,
                 map_string_nested_enum: Optional[Mapping[Text, TestAllTypesProto2.NestedEnum]]=...,
                 map_string_foreign_enum: Optional[Mapping[Text, ForeignEnumProto2]]=...,
                 oneof_uint32: Optional[int] = ...,
                 oneof_nested_message: Optional[TestAllTypesProto2.NestedMessage] = ...,
                 oneof_string: Optional[Text] = ...,
                 oneof_bytes: Optional[str] = ...,
                 oneof_bool: Optional[bool] = ...,
                 oneof_uint64: Optional[int] = ...,
                 oneof_float: Optional[float] = ...,
                 oneof_double: Optional[float] = ...,
                 oneof_enum: Optional[TestAllTypesProto2.NestedEnum] = ...,
                 data: Optional[TestAllTypesProto2.Data] = ...,
                 fieldname1: Optional[int] = ...,
                 field_name2: Optional[int] = ...,
                 _field_name3: Optional[int] = ...,
                 field__name4_: Optional[int] = ...,
                 field0name5: Optional[int] = ...,
                 field_0_name6: Optional[int] = ...,
                 fieldName7: Optional[int] = ...,
                 FieldName8: Optional[int] = ...,
                 field_Name9: Optional[int] = ...,
                 Field_Name10: Optional[int] = ...,
                 FIELD_NAME11: Optional[int] = ...,
                 FIELD_name12: Optional[int] = ...,
                 __field_name13: Optional[int] = ...,
                 __Field_name14: Optional[int] = ...,
                 field__name15: Optional[int] = ...,
                 field__Name16: Optional[int] = ...,
                 field_name17__: Optional[int] = ...,
                 Field_name18__: Optional[int] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> TestAllTypesProto2: ...


class ForeignMessageProto2(Message):
    c = ...  # type: int

    def __init__(self,
                 c: Optional[int] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> ForeignMessageProto2: ...



# Stubs for random
# Ron Murawski <ron@horizonchess.com>
# Updated by Jukka Lehtosalo

# based on http://docs.python.org/3.2/library/random.html

# ----- random classes -----

import _random
import sys
from typing import (
    Any, TypeVar, Sequence, List, Callable, AbstractSet, Union, Optional
)

_T = TypeVar('_T')

class Random(_random.Random):
    def __init__(self, x: Any = ...) -> None: ...
    def seed(self, a: Any = ..., version: int = ...) -> None: ...
    def getstate(self) -> tuple: ...
    def setstate(self, state: tuple) -> None: ...
    def getrandbits(self, k: int) -> int: ...
    def randrange(self, start: int, stop: Union[int, None] = ..., step: int = ...) -> int: ...
    def randint(self, a: int, b: int) -> int: ...
    def choice(self, seq: Sequence[_T]) -> _T: ...
    if sys.version_info >= (3, 6):
        def choices(self, population: Sequence[_T], weights: Optional[Sequence[float]] = ..., *, cum_weights: Optional[Sequence[float]] = ..., k: int = ...) -> List[_T]: ...
    def shuffle(self, x: List[Any], random: Union[Callable[[], float], None] = ...) -> None: ...
    def sample(self, population: Union[Sequence[_T], AbstractSet[_T]], k: int) -> List[_T]: ...
    def random(self) -> float: ...
    def uniform(self, a: float, b: float) -> float: ...
    def triangular(self, low: float = ..., high: float = ...,
                     mode: float = ...) -> float: ...
    def betavariate(self, alpha: float, beta: float) -> float: ...
    def expovariate(self, lambd: float) -> float: ...
    def gammavariate(self, alpha: float, beta: float) -> float: ...
    def gauss(self, mu: float, sigma: float) -> float: ...
    def lognormvariate(self, mu: float, sigma: float) -> float: ...
    def normalvariate(self, mu: float, sigma: float) -> float: ...
    def vonmisesvariate(self, mu: float, kappa: float) -> float: ...
    def paretovariate(self, alpha: float) -> float: ...
    def weibullvariate(self, alpha: float, beta: float) -> float: ...

# SystemRandom is not implemented for all OS's; good on Windows & Linux
class SystemRandom(Random):
    ...

# ----- random function stubs -----
def seed(a: Any = ..., version: int = ...) -> None: ...
def getstate() -> object: ...
def setstate(state: object) -> None: ...
def getrandbits(k: int) -> int: ...
def randrange(start: int, stop: Union[None, int] = ..., step: int = ...) -> int: ...
def randint(a: int, b: int) -> int: ...
def choice(seq: Sequence[_T]) -> _T: ...
if sys.version_info >= (3, 6):
    def choices(population: Sequence[_T], weights: Optional[Sequence[float]] = ..., *, cum_weights: Optional[Sequence[float]] = ..., k: int = ...) -> List[_T]: ...
def shuffle(x: List[Any], random: Union[Callable[[], float], None] = ...) -> None: ...
def sample(population: Union[Sequence[_T], AbstractSet[_T]], k: int) -> List[_T]: ...
def random() -> float: ...
def uniform(a: float, b: float) -> float: ...
def triangular(low: float = ..., high: float = ...,
               mode: float = ...) -> float: ...
def betavariate(alpha: float, beta: float) -> float: ...
def expovariate(lambd: float) -> float: ...
def gammavariate(alpha: float, beta: float) -> float: ...
def gauss(mu: float, sigma: float) -> float: ...
def lognormvariate(mu: float, sigma: float) -> float: ...
def normalvariate(mu: float, sigma: float) -> float: ...
def vonmisesvariate(mu: float, kappa: float) -> float: ...
def paretovariate(alpha: float) -> float: ...
def weibullvariate(alpha: float, beta: float) -> float: ...



# Stubs for pytz (Python 3.5)

import datetime
from typing import Optional, List, Set, Dict, Union

all_timezones = ...  # type: List
all_timezones_set = ...  # type: Set
common_timezones = ...  # type: List
common_timezones_set = ...  # type: Set
country_timezones = ...  # type: Dict
country_names = ...  # type: Dict


class _UTCclass(datetime.tzinfo):
    zone = ...  # type: str
    def fromutc(self, dt: datetime.datetime) -> datetime.datetime: ...
    def utcoffset(self, dt: Optional[datetime.datetime]) -> datetime.timedelta: ...
    def tzname(self, dt: Optional[datetime.datetime]) -> str: ...
    def dst(self, dt: Optional[datetime.datetime]) -> datetime.timedelta: ...
    def localize(self, dt: datetime.datetime, is_dst: bool = ...) -> datetime.datetime: ...
    def normalize(self, dt: datetime.datetime, is_dst: bool = ...) -> datetime.datetime: ...

utc = ...  # type: _UTCclass
UTC = ...  # type: _UTCclass


class _BaseTzInfo(datetime.tzinfo):
    zone = ...  # type: str

    def fromutc(self, dt: datetime.datetime) -> datetime.datetime: ...
    def localize(self, dt: datetime.datetime, is_dst: Optional[bool] = ...) -> datetime.datetime: ...
    def normalize(self, dt: datetime.datetime) -> datetime.datetime: ...


class _StaticTzInfo(_BaseTzInfo):
    def normalize(self, dt: datetime.datetime, is_dst: Optional[bool] = ...) -> datetime.datetime: ...


def timezone(zone: str) -> _BaseTzInfo: ...



# Stubs for getopt

# Based on http://docs.python.org/3.2/library/getopt.html

from typing import List, Tuple

def getopt(args: List[str], shortopts: str,
           longopts: List[str]=...) -> Tuple[List[Tuple[str, str]],
                                             List[str]]: ...

def gnu_getopt(args: List[str], shortopts: str,
               longopts: List[str]=...) -> Tuple[List[Tuple[str, str]],
                                                 List[str]]: ...

class GetoptError(Exception):
    msg = ...  # type: str
    opt = ...  # type: str

error = GetoptError



# Stubs for tempfile
# Ron Murawski <ron@horizonchess.com>

# based on http: //docs.python.org/3.3/library/tempfile.html
# Adapted for Python 2.7 by Michal Pokorny

# TODO: Don't use basestring. Use Union[str, bytes] or AnyStr for arguments.
#       Avoid using Union[str, bytes] for return values, as it implies that
#       an isinstance() check will often be required, which is inconvenient.

from typing import Tuple, IO, Union, AnyStr, Any, overload, Iterator, List, Type, Optional

import thread
import random

TMP_MAX = ...  # type: int
tempdir = ...  # type: str
template = ...  # type: str
_name_sequence = ...  # type: Optional[_RandomNameSequence]

class _RandomNameSequence:
    _rng = ...  # type: random.Random
    _rng_pid = ...  # type: int
    characters = ...  # type: str
    mutex = ...  # type: thread.LockType
    rng = ...  # type: random.Random
    def __iter__(self) -> "_RandomNameSequence": ...
    def next(self) -> str: ...
    # from os.path:
    def normcase(self, path: AnyStr) -> AnyStr: ...

class _TemporaryFileWrapper(IO[str]):
    close_called = ...  # type: bool
    delete = ...  # type: bool
    file = ...  # type: IO
    name = ...  # type: Any
    def __init__(self, file: IO, name, delete: bool = ...) -> None: ...
    def __del__(self) -> None: ...
    def __enter__(self) -> "_TemporaryFileWrapper": ...
    def __exit__(self, exc, value, tb) -> bool: ...
    def __getattr__(self, name: unicode) -> Any: ...
    def close(self) -> None: ...
    def unlink(self, path: unicode) -> None: ...

# TODO text files

def TemporaryFile(
    mode: Union[bytes, unicode] = ...,
    bufsize: int = ...,
    suffix: Union[bytes, unicode] = ...,
    prefix: Union[bytes, unicode] = ...,
    dir: Union[bytes, unicode] = ...
) -> _TemporaryFileWrapper:
    ...

def NamedTemporaryFile(
    mode: Union[bytes, unicode] = ...,
    bufsize: int = ...,
    suffix: Union[bytes, unicode] = ...,
    prefix: Union[bytes, unicode] = ...,
    dir: Union[bytes, unicode] = ...,
    delete: bool = ...
) -> _TemporaryFileWrapper:
    ...

def SpooledTemporaryFile(
    max_size: int = ...,
    mode: Union[bytes, unicode] = ...,
    buffering: int = ...,
    suffix: Union[bytes, unicode] = ...,
    prefix: Union[bytes, unicode] = ...,
    dir: Union[bytes, unicode] = ...
) -> _TemporaryFileWrapper:
    ...

class TemporaryDirectory:
    name = ...  # type: Any  # Can be str or unicode
    def __init__(self,
                 suffix: Union[bytes, unicode] = ...,
                 prefix: Union[bytes, unicode] = ...,
                 dir: Union[bytes, unicode] = ...) -> None: ...
    def cleanup(self) -> None: ...
    def __enter__(self) -> Any: ...  # Can be str or unicode
    def __exit__(self, type, value, traceback) -> bool: ...

@overload
def mkstemp() -> Tuple[int, str]: ...
@overload
def mkstemp(suffix: AnyStr = ..., prefix: AnyStr = ..., dir: Optional[AnyStr] = ...,
            text: bool = ...) -> Tuple[int, AnyStr]: ...
@overload
def mkdtemp() -> str: ...
@overload
def mkdtemp(suffix: AnyStr = ..., prefix: AnyStr = ..., dir: Optional[AnyStr] = ...) -> AnyStr: ...
@overload
def mktemp() -> str: ...
@overload
def mktemp(suffix: AnyStr = ..., prefix: AnyStr = ..., dir: Optional[AnyStr] = ...) -> AnyStr: ...
def gettempdir() -> str: ...
def gettempprefix() -> str: ...

def _candidate_tempdir_list() -> List[str]: ...
def _get_candidate_names() -> Optional[_RandomNameSequence]: ...
def _get_default_tempdir() -> str: ...



"""Expression type checker. This file is conceptually part of ExpressionChecker and TypeChecker."""

import re

from typing import cast, List, Tuple, Dict, Callable, Union, Optional

from mypy.types import (
    Type, AnyType, TupleType, Instance, UnionType, TypeOfAny
)
from mypy.nodes import (
    StrExpr, BytesExpr, UnicodeExpr, TupleExpr, DictExpr, Context, Expression, StarExpr
)
if False:
    # break import cycle only needed for mypy
    import mypy.checker
    import mypy.checkexpr
from mypy import messages
from mypy.messages import MessageBuilder

FormatStringExpr = Union[StrExpr, BytesExpr, UnicodeExpr]
Checkers = Tuple[Callable[[Expression], None], Callable[[Type], None]]


class ConversionSpecifier:
    def __init__(self, key: str, flags: str, width: str, precision: str, type: str) -> None:
        self.key = key
        self.flags = flags
        self.width = width
        self.precision = precision
        self.type = type

    def has_key(self) -> bool:
        return self.key is not None

    def has_star(self) -> bool:
        return self.width == '*' or self.precision == '*'


class StringFormatterChecker:
    """String interpolation/formatter type checker.

    This class works closely together with checker.ExpressionChecker.
    """

    # Some services are provided by a TypeChecker instance.
    chk = None  # type: mypy.checker.TypeChecker
    # This is shared with TypeChecker, but stored also here for convenience.
    msg = None  # type: MessageBuilder
    # Some services are provided by a ExpressionChecker instance.
    exprchk = None  # type: mypy.checkexpr.ExpressionChecker

    def __init__(self,
                 exprchk: 'mypy.checkexpr.ExpressionChecker',
                 chk: 'mypy.checker.TypeChecker',
                 msg: MessageBuilder) -> None:
        """Construct an expression type checker."""
        self.chk = chk
        self.exprchk = exprchk
        self.msg = msg

    # TODO: In Python 3, the bytes formatting has a more restricted set of options
    # compared to string formatting.
    def check_str_interpolation(self,
                                expr: FormatStringExpr,
                                replacements: Expression) -> Type:
        """Check the types of the 'replacements' in a string interpolation
        expression: str % replacements
        """
        specifiers = self.parse_conversion_specifiers(expr.value)
        has_mapping_keys = self.analyze_conversion_specifiers(specifiers, expr)
        if isinstance(expr, BytesExpr) and (3, 0) <= self.chk.options.python_version < (3, 5):
            self.msg.fail('Bytes formatting is only supported in Python 3.5 and later',
                          replacements)
            return AnyType(TypeOfAny.from_error)

        if has_mapping_keys is None:
            pass  # Error was reported
        elif has_mapping_keys:
            self.check_mapping_str_interpolation(specifiers, replacements, expr)
        else:
            self.check_simple_str_interpolation(specifiers, replacements, expr)

        if isinstance(expr, BytesExpr):
            return self.named_type('builtins.bytes')
        elif isinstance(expr, UnicodeExpr):
            return self.named_type('builtins.unicode')
        elif isinstance(expr, StrExpr):
            return self.named_type('builtins.str')
        else:
            assert False

    def parse_conversion_specifiers(self, format: str) -> List[ConversionSpecifier]:
        key_regex = r'(\(([^()]*)\))?'  # (optional) parenthesised sequence of characters
        flags_regex = r'([#0\-+ ]*)'  # (optional) sequence of flags
        width_regex = r'(\*|[1-9][0-9]*)?'  # (optional) minimum field width (* or numbers)
        precision_regex = r'(?:\.(\*|[0-9]+)?)?'  # (optional) . followed by * of numbers
        length_mod_regex = r'[hlL]?'  # (optional) length modifier (unused)
        type_regex = r'(.)?'  # conversion type
        regex = ('%' + key_regex + flags_regex + width_regex +
                 precision_regex + length_mod_regex + type_regex)
        specifiers = []  # type: List[ConversionSpecifier]
        for parens_key, key, flags, width, precision, type in re.findall(regex, format):
            if parens_key == '':
                key = None
            specifiers.append(ConversionSpecifier(key, flags, width, precision, type))
        return specifiers

    def analyze_conversion_specifiers(self, specifiers: List[ConversionSpecifier],
                                      context: Context) -> Optional[bool]:
        has_star = any(specifier.has_star() for specifier in specifiers)
        has_key = any(specifier.has_key() for specifier in specifiers)
        all_have_keys = all(
            specifier.has_key() or specifier.type == '%' for specifier in specifiers
        )

        if has_key and has_star:
            self.msg.string_interpolation_with_star_and_key(context)
            return None
        if has_key and not all_have_keys:
            self.msg.string_interpolation_mixing_key_and_non_keys(context)
            return None
        return has_key

    def check_simple_str_interpolation(self, specifiers: List[ConversionSpecifier],
                                       replacements: Expression, expr: FormatStringExpr) -> None:
        checkers = self.build_replacement_checkers(specifiers, replacements, expr)
        if checkers is None:
            return

        rhs_type = self.accept(replacements)
        rep_types = []  # type: List[Type]
        if isinstance(rhs_type, TupleType):
            rep_types = rhs_type.items
        elif isinstance(rhs_type, AnyType):
            return
        else:
            rep_types = [rhs_type]

        if len(checkers) > len(rep_types):
            self.msg.too_few_string_formatting_arguments(replacements)
        elif len(checkers) < len(rep_types):
            self.msg.too_many_string_formatting_arguments(replacements)
        else:
            if len(checkers) == 1:
                check_node, check_type = checkers[0]
                if isinstance(rhs_type, TupleType) and len(rhs_type.items) == 1:
                    check_type(rhs_type.items[0])
                else:
                    check_node(replacements)
            elif (isinstance(replacements, TupleExpr)
                  and not any(isinstance(item, StarExpr) for item in replacements.items)):
                for checks, rep_node in zip(checkers, replacements.items):
                    check_node, check_type = checks
                    check_node(rep_node)
            else:
                for checks, rep_type in zip(checkers, rep_types):
                    check_node, check_type = checks
                    check_type(rep_type)

    def check_mapping_str_interpolation(self, specifiers: List[ConversionSpecifier],
                                        replacements: Expression,
                                        expr: FormatStringExpr) -> None:
        if (isinstance(replacements, DictExpr) and
                all(isinstance(k, (StrExpr, BytesExpr))
                    for k, v in replacements.items)):
            mapping = {}  # type: Dict[str, Type]
            for k, v in replacements.items:
                key_str = cast(StrExpr, k).value
                mapping[key_str] = self.accept(v)

            for specifier in specifiers:
                if specifier.type == '%':
                    # %% is allowed in mappings, no checking is required
                    continue
                if specifier.key not in mapping:
                    self.msg.key_not_in_mapping(specifier.key, replacements)
                    return
                rep_type = mapping[specifier.key]
                expected_type = self.conversion_type(specifier.type, replacements, expr)
                if expected_type is None:
                    return
                self.chk.check_subtype(rep_type, expected_type, replacements,
                                       messages.INCOMPATIBLE_TYPES_IN_STR_INTERPOLATION,
                                       'expression has type',
                                       'placeholder with key \'%s\' has type' % specifier.key)
        else:
            rep_type = self.accept(replacements)
            any_type = AnyType(TypeOfAny.special_form)
            dict_type = self.chk.named_generic_type('builtins.dict',
                                                    [any_type, any_type])
            self.chk.check_subtype(rep_type, dict_type, replacements,
                                   messages.FORMAT_REQUIRES_MAPPING,
                                   'expression has type', 'expected type for mapping is')

    def build_replacement_checkers(self, specifiers: List[ConversionSpecifier],
                                   context: Context, expr: FormatStringExpr
                                   ) -> Optional[List[Checkers]]:
        checkers = []  # type: List[Checkers]
        for specifier in specifiers:
            checker = self.replacement_checkers(specifier, context, expr)
            if checker is None:
                return None
            checkers.extend(checker)
        return checkers

    def replacement_checkers(self, specifier: ConversionSpecifier, context: Context,
                             expr: FormatStringExpr) -> Optional[List[Checkers]]:
        """Returns a list of tuples of two functions that check whether a replacement is
        of the right type for the specifier. The first functions take a node and checks
        its type in the right type context. The second function just checks a type.
        """
        checkers = []  # type: List[Checkers]

        if specifier.width == '*':
            checkers.append(self.checkers_for_star(context))
        if specifier.precision == '*':
            checkers.append(self.checkers_for_star(context))
        if specifier.type == 'c':
            c = self.checkers_for_c_type(specifier.type, context, expr)
            if c is None:
                return None
            checkers.append(c)
        elif specifier.type != '%':
            c = self.checkers_for_regular_type(specifier.type, context, expr)
            if c is None:
                return None
            checkers.append(c)
        return checkers

    def checkers_for_star(self, context: Context) -> Checkers:
        """Returns a tuple of check functions that check whether, respectively,
        a node or a type is compatible with a star in a conversion specifier
        """
        expected = self.named_type('builtins.int')

        def check_type(type: Type) -> None:
            expected = self.named_type('builtins.int')
            self.chk.check_subtype(type, expected, context, '* wants int')

        def check_expr(expr: Expression) -> None:
            type = self.accept(expr, expected)
            check_type(type)

        return check_expr, check_type

    def checkers_for_regular_type(self, type: str,
                                  context: Context,
                                  expr: FormatStringExpr) -> Optional[Checkers]:
        """Returns a tuple of check functions that check whether, respectively,
        a node or a type is compatible with 'type'. Return None in case of an
        """
        expected_type = self.conversion_type(type, context, expr)
        if expected_type is None:
            return None

        def check_type(type: Type) -> None:
            assert expected_type is not None
            self.chk.check_subtype(type, expected_type, context,
                              messages.INCOMPATIBLE_TYPES_IN_STR_INTERPOLATION,
                              'expression has type', 'placeholder has type')

        def check_expr(expr: Expression) -> None:
            type = self.accept(expr, expected_type)
            check_type(type)

        return check_expr, check_type

    def checkers_for_c_type(self, type: str,
                            context: Context,
                            expr: FormatStringExpr) -> Optional[Checkers]:
        """Returns a tuple of check functions that check whether, respectively,
        a node or a type is compatible with 'type' that is a character type
        """
        expected_type = self.conversion_type(type, context, expr)
        if expected_type is None:
            return None

        def check_type(type: Type) -> None:
            assert expected_type is not None
            self.chk.check_subtype(type, expected_type, context,
                              messages.INCOMPATIBLE_TYPES_IN_STR_INTERPOLATION,
                              'expression has type', 'placeholder has type')

        def check_expr(expr: Expression) -> None:
            """int, or str with length 1"""
            type = self.accept(expr, expected_type)
            if isinstance(expr, (StrExpr, BytesExpr)) and len(cast(StrExpr, expr).value) != 1:
                self.msg.requires_int_or_char(context)
            check_type(type)

        return check_expr, check_type

    def conversion_type(self, p: str, context: Context, expr: FormatStringExpr) -> Optional[Type]:
        """Return the type that is accepted for a string interpolation
        conversion specifier type.

        Note that both Python's float (e.g. %f) and integer (e.g. %d)
        specifier types accept both float and integers.
        """
        if p == 'b':
            if self.chk.options.python_version < (3, 5):
                self.msg.fail("Format character 'b' is only supported in Python 3.5 and later",
                              context)
                return None
            if not isinstance(expr, BytesExpr):
                self.msg.fail("Format character 'b' is only supported on bytes patterns", context)
                return None
            return self.named_type('builtins.bytes')
        elif p == 'a':
            if self.chk.options.python_version < (3, 0):
                self.msg.fail("Format character 'a' is only supported in Python 3", context)
                return None
            # todo: return type object?
            return AnyType(TypeOfAny.special_form)
        elif p in ['s', 'r']:
            return AnyType(TypeOfAny.special_form)
        elif p in ['d', 'i', 'o', 'u', 'x', 'X',
                   'e', 'E', 'f', 'F', 'g', 'G']:
            return UnionType([self.named_type('builtins.int'),
                              self.named_type('builtins.float')])
        elif p in ['c']:
            return UnionType([self.named_type('builtins.int'),
                              self.named_type('builtins.float'),
                              self.named_type('builtins.str')])
        else:
            self.msg.unsupported_placeholder(p, context)
            return None

    #
    # Helpers
    #

    def named_type(self, name: str) -> Instance:
        """Return an instance type with type given by the name and no type
        arguments. Alias for TypeChecker.named_type.
        """
        return self.chk.named_type(name)

    def accept(self, expr: Expression, context: Optional[Type] = None) -> Type:
        """Type check a node. Alias for TypeChecker.accept."""
        return self.chk.expr_checker.accept(expr, context)



"""Tests for mypy incremental error output."""
from typing import List, Callable, Optional

import os

from mypy import defaults, build
from mypy.test.helpers import assert_string_arrays_equal
from mypy.test.data import DataDrivenTestCase, DataSuite
from mypy.build import BuildSource
from mypy.errors import CompileError
from mypy.options import Options
from mypy.nodes import CallExpr, StrExpr
from mypy.types import Type


class ErrorStreamSuite(DataSuite):
    required_out_section = True
    base_path = '.'
    files = ['errorstream.test']

    def run_case(self, testcase: DataDrivenTestCase) -> None:
        test_error_stream(testcase)


def test_error_stream(testcase: DataDrivenTestCase) -> None:
    """Perform a single error streaming test case.

    The argument contains the description of the test case.
    """
    options = Options()
    options.show_traceback = True

    logged_messages = []  # type: List[str]

    def flush_errors(msgs: List[str], serious: bool) -> None:
        if msgs:
            logged_messages.append('==== Errors flushed ====')
            logged_messages.extend(msgs)

    sources = [BuildSource('main', '__main__', '\n'.join(testcase.input))]
    try:
        build.build(sources=sources,
                    options=options,
                    flush_errors=flush_errors)
    except CompileError as e:
        assert e.messages == []

    assert_string_arrays_equal(testcase.output, logged_messages,
                               'Invalid output ({}, line {})'.format(
                                   testcase.file, testcase.line))



from typing import Any, Optional

class Rule:
    id = ...  # type: Any
    prefix = ...  # type: Any
    status = ...  # type: Any
    expiration = ...  # type: Any
    transition = ...  # type: Any
    def __init__(self, id: Optional[Any] = ..., prefix: Optional[Any] = ..., status: Optional[Any] = ..., expiration: Optional[Any] = ..., transition: Optional[Any] = ...) -> None: ...
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...
    def to_xml(self): ...

class Expiration:
    days = ...  # type: Any
    date = ...  # type: Any
    def __init__(self, days: Optional[Any] = ..., date: Optional[Any] = ...) -> None: ...
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...
    def to_xml(self): ...

class Transition:
    days = ...  # type: Any
    date = ...  # type: Any
    storage_class = ...  # type: Any
    def __init__(self, days: Optional[Any] = ..., date: Optional[Any] = ..., storage_class: Optional[Any] = ...) -> None: ...
    def to_xml(self): ...

class Transitions(list):
    transition_properties = ...  # type: int
    current_transition_property = ...  # type: int
    temp_days = ...  # type: Any
    temp_date = ...  # type: Any
    temp_storage_class = ...  # type: Any
    def __init__(self) -> None: ...
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...
    def to_xml(self): ...
    def add_transition(self, days: Optional[Any] = ..., date: Optional[Any] = ..., storage_class: Optional[Any] = ...): ...
    @property
    def days(self): ...
    @property
    def date(self): ...
    @property
    def storage_class(self): ...

class Lifecycle(list):
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...
    def to_xml(self): ...
    def add_rule(self, id: Optional[Any] = ..., prefix: str = ..., status: str = ..., expiration: Optional[Any] = ..., transition: Optional[Any] = ...): ...



from typing import Any

class ReloaderLoop:
    name = ...  # type: Any
    extra_files = ...  # type: Any
    interval = ...  # type: Any
    def __init__(self, extra_files=None, interval=1): ...
    def run(self): ...
    def restart_with_reloader(self): ...
    def trigger_reload(self, filename): ...
    def log_reload(self, filename): ...

class StatReloaderLoop(ReloaderLoop):
    name = ...  # type: Any
    def run(self): ...

class WatchdogReloaderLoop(ReloaderLoop):
    observable_paths = ...  # type: Any
    name = ...  # type: Any
    observer_class = ...  # type: Any
    event_handler = ...  # type: Any
    should_reload = ...  # type: Any
    def __init__(self, *args, **kwargs): ...
    def trigger_reload(self, filename): ...
    def run(self): ...

reloader_loops = ...  # type: Any

def run_with_reloader(main_func, extra_files=None, interval=1, reloader_type=''): ...



import sys
from typing import Any, BinaryIO, IO, Optional, Union

if sys.version_info >= (3, 6):
    from os import PathLike
    _PathOrFile = Union[str, bytes, IO[Any], PathLike[Any]]
elif sys.version_info >= (3, 3):
    _PathOrFile = Union[str, bytes, IO[Any]]
else:
    _PathOrFile = str

def compress(data: bytes, compresslevel: int = ...) -> bytes: ...
def decompress(data: bytes) -> bytes: ...

if sys.version_info >= (3, 3):
    def open(filename: _PathOrFile,
             mode: str = ...,
             compresslevel: int = ...,
             encoding: Optional[str] = ...,
             errors: Optional[str] = ...,
             newline: Optional[str] = ...) -> IO[Any]: ...

class BZ2File(BinaryIO):
    def __init__(self,
                 filename: _PathOrFile,
                 mode: str = ...,
                 buffering: Optional[Any] = ...,
                 compresslevel: int = ...) -> None: ...

class BZ2Compressor(object):
    def __init__(self, compresslevel: int = ...) -> None: ...
    def compress(self, data: bytes) -> bytes: ...
    def flush(self) -> bytes: ...

class BZ2Decompressor(object):
    if sys.version_info >= (3, 5):
        def decompress(self, data: bytes, max_length: int = ...) -> bytes: ...
    else:
        def decompress(self, data: bytes) -> bytes: ...
    if sys.version_info >= (3, 3):
        @property
        def eof(self) -> bool: ...
    if sys.version_info >= (3, 5):
        @property
        def needs_input(self) -> bool: ...
    @property
    def unused_data(self) -> bytes: ...



from typing import TypeVar, List, Iterable, Any, Callable, Optional

_T = TypeVar('_T')

def cmp_lt(x, y) -> bool: ...
def heappush(heap: List[_T], item: _T) -> None: ...
def heappop(heap: List[_T]) -> _T:
    raise IndexError()  # if heap is empty
def heappushpop(heap: List[_T], item: _T) -> _T: ...
def heapify(x: List[_T]) -> None: ...
def heapreplace(heap: List[_T], item: _T) -> _T:
    raise IndexError()  # if heap is empty
def merge(*iterables: Iterable[_T]) -> Iterable[_T]: ...
def nlargest(n: int, iterable: Iterable[_T],
             key: Optional[Callable[[_T], Any]] = ...) -> List[_T]: ...
def nsmallest(n: int, iterable: Iterable[_T]) -> List[_T]: ...



"""Fix up various things after deserialization."""

from typing import Any, Dict, Optional

from mypy.nodes import (
    MypyFile, SymbolNode, SymbolTable, SymbolTableNode,
    TypeInfo, FuncDef, OverloadedFuncDef, Decorator, Var,
    TypeVarExpr, ClassDef, Block,
    LDEF, MDEF, GDEF, TYPE_ALIAS
)
from mypy.types import (
    CallableType, EllipsisType, Instance, Overloaded, TupleType, TypedDictType,
    TypeList, TypeVarType, UnboundType, UnionType, TypeVisitor,
    TypeType, NOT_READY
)
from mypy.visitor import NodeVisitor


# N.B: we do a quick_and_dirty fixup in both quick_and_dirty mode and
# when fixing up a fine-grained incremental cache load (since there may
# be cross-refs into deleted modules)
def fixup_module(tree: MypyFile, modules: Dict[str, MypyFile],
                 quick_and_dirty: bool) -> None:
    node_fixer = NodeFixer(modules, quick_and_dirty)
    node_fixer.visit_symbol_table(tree.names)


# TODO: Fix up .info when deserializing, i.e. much earlier.
class NodeFixer(NodeVisitor[None]):
    current_info = None  # type: Optional[TypeInfo]

    def __init__(self, modules: Dict[str, MypyFile], quick_and_dirty: bool) -> None:
        self.modules = modules
        self.quick_and_dirty = quick_and_dirty
        self.type_fixer = TypeFixer(self.modules, quick_and_dirty)

    # NOTE: This method isn't (yet) part of the NodeVisitor API.
    def visit_type_info(self, info: TypeInfo) -> None:
        save_info = self.current_info
        try:
            self.current_info = info
            if info.defn:
                info.defn.accept(self)
            if info.names:
                self.visit_symbol_table(info.names)
            if info.bases:
                for base in info.bases:
                    base.accept(self.type_fixer)
            if info._promote:
                info._promote.accept(self.type_fixer)
            if info.tuple_type:
                info.tuple_type.accept(self.type_fixer)
            if info.typeddict_type:
                info.typeddict_type.accept(self.type_fixer)
            if info.declared_metaclass:
                info.declared_metaclass.accept(self.type_fixer)
            if info.metaclass_type:
                info.metaclass_type.accept(self.type_fixer)
            if info._mro_refs:
                info.mro = [lookup_qualified_typeinfo(self.modules, name, self.quick_and_dirty)
                            for name in info._mro_refs]
                info._mro_refs = None
        finally:
            self.current_info = save_info

    # NOTE: This method *definitely* isn't part of the NodeVisitor API.
    def visit_symbol_table(self, symtab: SymbolTable) -> None:
        # Copy the items because we may mutate symtab.
        for key, value in list(symtab.items()):
            cross_ref = value.cross_ref
            if cross_ref is not None:  # Fix up cross-reference.
                value.cross_ref = None
                if cross_ref in self.modules:
                    value.node = self.modules[cross_ref]
                else:
                    stnode = lookup_qualified_stnode(self.modules, cross_ref,
                                                     self.quick_and_dirty)
                    if stnode is not None:
                        value.node = stnode.node
                        value.type_override = stnode.type_override
                        if (self.quick_and_dirty and value.kind == TYPE_ALIAS and
                                stnode.type_override is None):
                            value.type_override = Instance(stale_info(self.modules), [])
                        value.alias_tvars = stnode.alias_tvars or []
                    elif not self.quick_and_dirty:
                        assert stnode is not None, "Could not find cross-ref %s" % (cross_ref,)
                    else:
                        # We have a missing crossref in quick mode, need to put something
                        value.node = stale_info(self.modules)
                        if value.kind == TYPE_ALIAS:
                            value.type_override = Instance(stale_info(self.modules), [])
            else:
                if isinstance(value.node, TypeInfo):
                    # TypeInfo has no accept().  TODO: Add it?
                    self.visit_type_info(value.node)
                elif value.node is not None:
                    value.node.accept(self)
                if value.type_override is not None:
                    value.type_override.accept(self.type_fixer)

    def visit_func_def(self, func: FuncDef) -> None:
        if self.current_info is not None:
            func.info = self.current_info
        if func.type is not None:
            func.type.accept(self.type_fixer)

    def visit_overloaded_func_def(self, o: OverloadedFuncDef) -> None:
        if self.current_info is not None:
            o.info = self.current_info
        if o.type:
            o.type.accept(self.type_fixer)
        for item in o.items:
            item.accept(self)
        if o.impl:
            o.impl.accept(self)

    def visit_decorator(self, d: Decorator) -> None:
        if self.current_info is not None:
            d.var.info = self.current_info
        if d.func:
            d.func.accept(self)
        if d.var:
            d.var.accept(self)
        for node in d.decorators:
            node.accept(self)

    def visit_class_def(self, c: ClassDef) -> None:
        for v in c.type_vars:
            for value in v.values:
                value.accept(self.type_fixer)
            v.upper_bound.accept(self.type_fixer)

    def visit_type_var_expr(self, tv: TypeVarExpr) -> None:
        for value in tv.values:
            value.accept(self.type_fixer)
        tv.upper_bound.accept(self.type_fixer)

    def visit_var(self, v: Var) -> None:
        if self.current_info is not None:
            v.info = self.current_info
        if v.type is not None:
            v.type.accept(self.type_fixer)


class TypeFixer(TypeVisitor[None]):
    def __init__(self, modules: Dict[str, MypyFile], quick_and_dirty: bool) -> None:
        self.modules = modules
        self.quick_and_dirty = quick_and_dirty

    def visit_instance(self, inst: Instance) -> None:
        # TODO: Combine Instances that are exactly the same?
        type_ref = inst.type_ref
        if type_ref is None:
            return  # We've already been here.
        inst.type_ref = None
        inst.type = lookup_qualified_typeinfo(self.modules, type_ref, self.quick_and_dirty)
        # TODO: Is this needed or redundant?
        # Also fix up the bases, just in case.
        for base in inst.type.bases:
            if base.type is NOT_READY:
                base.accept(self)
        for a in inst.args:
            a.accept(self)

    def visit_any(self, o: Any) -> None:
        pass  # Nothing to descend into.

    def visit_callable_type(self, ct: CallableType) -> None:
        if ct.fallback:
            ct.fallback.accept(self)
        for argt in ct.arg_types:
            # argt may be None, e.g. for __self in NamedTuple constructors.
            if argt is not None:
                argt.accept(self)
        if ct.ret_type is not None:
            ct.ret_type.accept(self)
        for v in ct.variables:
            if v.values:
                for val in v.values:
                    val.accept(self)
            v.upper_bound.accept(self)
        for arg in ct.bound_args:
            if arg:
                arg.accept(self)

    def visit_overloaded(self, t: Overloaded) -> None:
        for ct in t.items():
            ct.accept(self)

    def visit_deleted_type(self, o: Any) -> None:
        pass  # Nothing to descend into.

    def visit_none_type(self, o: Any) -> None:
        pass  # Nothing to descend into.

    def visit_uninhabited_type(self, o: Any) -> None:
        pass  # Nothing to descend into.

    def visit_partial_type(self, o: Any) -> None:
        raise RuntimeError("Shouldn't get here", o)

    def visit_tuple_type(self, tt: TupleType) -> None:
        if tt.items:
            for it in tt.items:
                it.accept(self)
        if tt.fallback is not None:
            tt.fallback.accept(self)

    def visit_typeddict_type(self, tdt: TypedDictType) -> None:
        if tdt.items:
            for it in tdt.items.values():
                it.accept(self)
        if tdt.fallback is not None:
            tdt.fallback.accept(self)

    def visit_type_var(self, tvt: TypeVarType) -> None:
        if tvt.values:
            for vt in tvt.values:
                vt.accept(self)
        if tvt.upper_bound is not None:
            tvt.upper_bound.accept(self)

    def visit_unbound_type(self, o: UnboundType) -> None:
        for a in o.args:
            a.accept(self)

    def visit_union_type(self, ut: UnionType) -> None:
        if ut.items:
            for it in ut.items:
                it.accept(self)

    def visit_void(self, o: Any) -> None:
        pass  # Nothing to descend into.

    def visit_type_type(self, t: TypeType) -> None:
        t.item.accept(self)


def lookup_qualified_typeinfo(modules: Dict[str, MypyFile], name: str,
                              quick_and_dirty: bool) -> TypeInfo:
    node = lookup_qualified(modules, name, quick_and_dirty)
    if isinstance(node, TypeInfo):
        return node
    else:
        # Looks like a missing TypeInfo in quick mode, put something there
        assert quick_and_dirty, "Should never get here in normal mode"
        return stale_info(modules)


def lookup_qualified(modules: Dict[str, MypyFile], name: str,
                     quick_and_dirty: bool) -> Optional[SymbolNode]:
    stnode = lookup_qualified_stnode(modules, name, quick_and_dirty)
    if stnode is None:
        return None
    else:
        return stnode.node


def lookup_qualified_stnode(modules: Dict[str, MypyFile], name: str,
                            quick_and_dirty: bool) -> Optional[SymbolTableNode]:
    head = name
    rest = []
    while True:
        if '.' not in head:
            if not quick_and_dirty:
                assert '.' in head, "Cannot find %s" % (name,)
            return None
        head, tail = head.rsplit('.', 1)
        rest.append(tail)
        mod = modules.get(head)
        if mod is not None:
            break
    names = mod.names
    while True:
        if not rest:
            if not quick_and_dirty:
                assert rest, "Cannot find %s" % (name,)
            return None
        key = rest.pop()
        if key not in names:
            if not quick_and_dirty:
                assert key in names, "Cannot find %s for %s" % (key, name)
            return None
        stnode = names[key]
        if not rest:
            return stnode
        node = stnode.node
        # In fine-grained mode, could be a cross-reference to a deleted module
        if node is None:
            if not quick_and_dirty:
                assert node, "Cannot find %s" % (name,)
            return None
        assert isinstance(node, TypeInfo)
        names = node.names


def stale_info(modules: Dict[str, MypyFile]) -> TypeInfo:
    suggestion = "<stale cache: consider running mypy without --quick>"
    dummy_def = ClassDef(suggestion, Block([]))
    dummy_def.fullname = suggestion

    info = TypeInfo(SymbolTable(), dummy_def, "<stale>")
    obj_type = lookup_qualified(modules, 'builtins.object', False)
    assert isinstance(obj_type, TypeInfo)
    info.bases = [Instance(obj_type, [])]
    info.mro = [info, obj_type]
    return info



# Stubs for email.feedparser (Python 3.4)

from typing import Callable
import sys
from email.message import Message
if sys.version_info >= (3, 3):
    from email.policy import Policy

class FeedParser:
    if sys.version_info >= (3, 3):
        def __init__(self, _factory: Callable[[], Message] = ..., *,
                     policy: Policy = ...) -> None: ...
    else:
        def __init__(self,
                     _factory: Callable[[], Message] = ...) -> None: ...
    def feed(self, data: str) -> None: ...
    def close(self) -> Message: ...

if sys.version_info >= (3, 2):
    class BytesFeedParser:
        if sys.version_info >= (3, 3):
            def __init__(self, _factory: Callable[[], Message] = ..., *,
                         policy: Policy = ...) -> None: ...
        else:
            def __init__(self,
                         _factory: Callable[[], Message] = ...) -> None: ...
        def feed(self, data: str) -> None: ...
        def close(self) -> Message: ...



from typing import (Any, TypeVar, Set, Dict, List, TextIO, Union, Tuple, Generic, Callable,
                    Coroutine, Generator, Iterable, Awaitable, overload, Sequence, Iterator,
                    Optional)
import concurrent.futures
from .events import AbstractEventLoop
from .futures import Future

__all__: List[str]

_T = TypeVar('_T')
_T1 = TypeVar('_T1')
_T2 = TypeVar('_T2')
_T3 = TypeVar('_T3')
_T4 = TypeVar('_T4')
_T5 = TypeVar('_T5')
_FutureT = Union[Future[_T], Generator[Any, None, _T], Awaitable[_T]]

FIRST_EXCEPTION = 'FIRST_EXCEPTION'
FIRST_COMPLETED = 'FIRST_COMPLETED'
ALL_COMPLETED = 'ALL_COMPLETED'

def as_completed(fs: Sequence[_FutureT[_T]], *, loop: AbstractEventLoop = ...,
                 timeout: Optional[float] = ...) -> Iterator[Generator[Any, None, _T]]: ...
def ensure_future(coro_or_future: _FutureT[_T],
                  *, loop: AbstractEventLoop = ...) -> Future[_T]: ...
async = ensure_future
@overload
def gather(coro_or_future1: _FutureT[_T1],
           *, loop: AbstractEventLoop = ..., return_exceptions: bool = ...) -> Future[Tuple[_T1]]: ...
@overload
def gather(coro_or_future1: _FutureT[_T1], coro_or_future2: _FutureT[_T2],
           *, loop: AbstractEventLoop = ..., return_exceptions: bool = ...) -> Future[Tuple[_T1, _T2]]: ...
@overload
def gather(coro_or_future1: _FutureT[_T1], coro_or_future2: _FutureT[_T2], coro_or_future3: _FutureT[_T3],
           *, loop: AbstractEventLoop = ..., return_exceptions: bool = ...) -> Future[Tuple[_T1, _T2, _T3]]: ...
@overload
def gather(coro_or_future1: _FutureT[_T1], coro_or_future2: _FutureT[_T2], coro_or_future3: _FutureT[_T3],
           coro_or_future4: _FutureT[_T4],
           *, loop: AbstractEventLoop = ..., return_exceptions: bool = ...) -> Future[Tuple[_T1, _T2, _T3, _T4]]: ...
@overload
def gather(coro_or_future1: _FutureT[_T1], coro_or_future2: _FutureT[_T2], coro_or_future3: _FutureT[_T3],
           coro_or_future4: _FutureT[_T4], coro_or_future5: _FutureT[_T5],
           *, loop: AbstractEventLoop = ..., return_exceptions: bool = ...) -> Future[Tuple[_T1, _T2, _T3, _T4, _T5]]: ...
@overload
def gather(coro_or_future1: _FutureT[Any], coro_or_future2: _FutureT[Any], coro_or_future3: _FutureT[Any],
           coro_or_future4: _FutureT[Any], coro_or_future5: _FutureT[Any], coro_or_future6: _FutureT[Any],
           *coros_or_futures: _FutureT[Any],
           loop: AbstractEventLoop = ..., return_exceptions: bool = ...) -> Future[Tuple[Any, ...]]: ...
def run_coroutine_threadsafe(coro: _FutureT[_T],
                             loop: AbstractEventLoop) -> concurrent.futures.Future[_T]: ...
def shield(arg: _FutureT[_T], *, loop: AbstractEventLoop = ...) -> Future[_T]: ...
def sleep(delay: float, result: _T = ..., loop: AbstractEventLoop = ...) -> Future[_T]: ...
def wait(fs: Iterable[_FutureT[_T]], *, loop: AbstractEventLoop = ...,
    timeout: Optional[float] = ...,
         return_when: str = ...) -> Future[Tuple[Set[Future[_T]], Set[Future[_T]]]]: ...
def wait_for(fut: _FutureT[_T], timeout: Optional[float],
             *, loop: AbstractEventLoop = ...) -> Future[_T]: ...

class Task(Future[_T], Generic[_T]):
    _all_tasks = ...  # type: Set[Task]
    _current_tasks = ...  # type: Dict[AbstractEventLoop, Task]
    @classmethod
    def current_task(cls, loop: AbstractEventLoop = ...) -> Task: ...
    @classmethod
    def all_tasks(cls, loop: AbstractEventLoop = ...) -> Set[Task]: ...

    # Can't use a union, see mypy issue  #1873.
    @overload
    def __init__(self, coro: Generator[Any, None, _T],
                 *, loop: AbstractEventLoop = ...) -> None: ...
    @overload
    def __init__(self, coro: Awaitable[_T], *, loop: AbstractEventLoop = ...) -> None: ...

    def __repr__(self) -> str: ...
    def get_stack(self, *, limit: int = ...) -> List[Any]: ...  # return List[stackframe]
    def print_stack(self, *, limit: int = ..., file: TextIO = ...) -> None: ...
    def cancel(self) -> bool: ...
    def _step(self, value: Any = ..., exc: Exception = ...) -> None: ...
    def _wakeup(self, future: Future[Any]) -> None: ...



from typing import AnyStr, List

class _unicode: ...

def commonprefix(list: List[AnyStr]) -> AnyStr: ...
def exists(path: unicode) -> bool: ...
def getatime(path: unicode) -> float: ...
def getmtime(path: unicode) -> float: ...
def getctime(path: unicode) -> float: ...
def getsize(path: unicode) -> int: ...
def isfile(path: unicode) -> bool: ...
def isdir(path: unicode) -> bool: ...



from typing import Any, Optional, Union, IO, Text, Tuple, List
import datetime
from ._common import tzname_in_python2 as tzname_in_python2, _tzinfo as _tzinfo
from ._common import tzrangebase as tzrangebase, enfold as enfold
from ..relativedelta import relativedelta

_FileObj = Union[str, Text, IO[str], IO[Text]]

ZERO = ...  # type: datetime.timedelta
EPOCH = ...  # type: datetime.datetime
EPOCHORDINAL = ...  # type: int

class tzutc(datetime.tzinfo):
    def utcoffset(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def dst(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def tzname(self, dt: Optional[datetime.datetime]) -> str: ...
    def is_ambiguous(self, dt: Optional[datetime.datetime]) -> bool: ...
    def __eq__(self, other): ...
    __hash__ = ...  # type: Any
    def __ne__(self, other): ...
    __reduce__ = ...  # type: Any

class tzoffset(datetime.tzinfo):
    def __init__(self, name, offset) -> None: ...
    def utcoffset(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def dst(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def is_ambiguous(self, dt: Optional[datetime.datetime]) -> bool: ...
    def tzname(self, dt: Optional[datetime.datetime]) -> str: ...
    def __eq__(self, other): ...
    __hash__ = ...  # type: Any
    def __ne__(self, other): ...
    __reduce__ = ...  # type: Any

class tzlocal(_tzinfo):
    def __init__(self) -> None: ...
    def utcoffset(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def dst(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def tzname(self, dt: Optional[datetime.datetime]) -> str: ...
    def is_ambiguous(self, dt: Optional[datetime.datetime]) -> bool: ...
    def __eq__(self, other): ...
    __hash__ = ...  # type: Any
    def __ne__(self, other): ...
    __reduce__ = ...  # type: Any

class _ttinfo:
    def __init__(self) -> None: ...
    def __eq__(self, other): ...
    __hash__ = ...  # type: Any
    def __ne__(self, other): ...

class tzfile(_tzinfo):
    def __init__(self, fileobj: _FileObj, filename: Optional[Text] = ...) -> None: ...
    def is_ambiguous(self, dt: Optional[datetime.datetime], idx: Optional[int] = ...) -> bool: ...
    def utcoffset(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def dst(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def tzname(self, dt: Optional[datetime.datetime]) -> str: ...
    def __eq__(self, other): ...
    __hash__ = ...  # type: Any
    def __ne__(self, other): ...
    def __reduce__(self): ...
    def __reduce_ex__(self, protocol): ...

class tzrange(tzrangebase):
    hasdst = ...  # type: bool
    def __init__(self, stdabbr: Text, stdoffset: Union[int, datetime.timedelta, None] = ..., dstabbr: Optional[Text] = ..., dstoffset: Union[int, datetime.timedelta, None] = ..., start: Optional[relativedelta] = ..., end: Optional[relativedelta] = ...) -> None: ...
    def transitions(self, year: int) -> Tuple[datetime.datetime, datetime.datetime]: ...
    def __eq__(self, other): ...

class tzstr(tzrange):
    hasdst = ...  # type: bool
    def __init__(self, s: Union[bytes, _FileObj], posix_offset: bool = ...) -> None: ...

class tzical:
    def __init__(self, fileobj: _FileObj) -> None: ...
    def keys(self): ...
    def get(self, tzid: Optional[Any] = ...): ...

TZFILES = ...  # type: List[str]
TZPATHS = ...  # type: List[str]

def gettz(name: Optional[Text] = ...) -> Optional[datetime.tzinfo]: ...
def datetime_exists(dt: datetime.datetime, tz: Optional[datetime.tzinfo] = ...) -> bool: ...
def datetime_ambiguous(dt: datetime.datetime, tz: Optional[datetime.tzinfo] = ...) -> bool: ...



"""Self check mypy package"""
import sys
from typing import List

import pytest  # type: ignore

from mypy.test.helpers import Suite
from mypy.api import run


class SelfCheckSuite(Suite):
    def test_mypy_package(self) -> None:
        run_mypy(['-p', 'mypy'])

    def test_testrunner(self) -> None:
        run_mypy(['runtests.py', 'waiter.py'])


def run_mypy(args: List[str]) -> None:
    __tracebackhide__ = True
    outval, errval, status = run(args + ['--config-file', 'mypy_self_check.ini',
                                         '--show-traceback',
                                         '--no-site-packages'])
    if status != 0:
        sys.stdout.write(outval)
        errval = '\n'.join(line for line in errval.split('\n')
                           if 'mypy_self_check.ini' not in line)
        sys.stderr.write(errval)
        pytest.fail(msg="Self check failed", pytrace=False)



from typing import Callable, Any, Tuple, Union
from types import FrameType

SIG_DFL = ...  # type: int
SIG_IGN = ...  # type: int
ITIMER_REAL = ...  # type: int
ITIMER_VIRTUAL = ...  # type: int
ITIMER_PROF = ...  # type: int

SIGABRT = ...  # type: int
SIGALRM = ...  # type: int
SIGBUS = ...  # type: int
SIGCHLD = ...  # type: int
SIGCLD = ...  # type: int
SIGCONT = ...  # type: int
SIGFPE = ...  # type: int
SIGHUP = ...  # type: int
SIGILL = ...  # type: int
SIGINT = ...  # type: int
SIGIO = ...  # type: int
SIGIOT = ...  # type: int
SIGKILL = ...  # type: int
SIGPIPE = ...  # type: int
SIGPOLL = ...  # type: int
SIGPROF = ...  # type: int
SIGPWR = ...  # type: int
SIGQUIT = ...  # type: int
SIGRTMAX = ...  # type: int
SIGRTMIN = ...  # type: int
SIGSEGV = ...  # type: int
SIGSTOP = ...  # type: int
SIGSYS = ...  # type: int
SIGTERM = ...  # type: int
SIGTRAP = ...  # type: int
SIGTSTP = ...  # type: int
SIGTTIN = ...  # type: int
SIGTTOU = ...  # type: int
SIGURG = ...  # type: int
SIGUSR1 = ...  # type: int
SIGUSR2 = ...  # type: int
SIGVTALRM = ...  # type: int
SIGWINCH = ...  # type: int
SIGXCPU = ...  # type: int
SIGXFSZ = ...  # type: int
NSIG = ...  # type: int

class ItimerError(IOError): ...

_HANDLER = Union[Callable[[int, FrameType], None], int, None]

def alarm(time: int) -> int: ...
def getsignal(signalnum: int) -> _HANDLER: ...
def pause() -> None: ...
def setitimer(which: int, seconds: float, interval: float = ...) -> Tuple[float, float]: ...
def getitimer(which: int) -> Tuple[float, float]: ...
def set_wakeup_fd(fd: int) -> int: ...
def siginterrupt(signalnum: int, flag: bool) -> None:
    raise RuntimeError()
def signal(signalnum: int, handler: _HANDLER) -> _HANDLER:
    raise RuntimeError()
def default_int_handler(signum: int, frame: FrameType) -> None:
    raise KeyboardInterrupt()



from typing import Any

SALT_CHARS = ...  # type: Any
DEFAULT_PBKDF2_ITERATIONS = ...  # type: Any

def pbkdf2_hex(data, salt, iterations=..., keylen=None, hashfunc=None): ...
def pbkdf2_bin(data, salt, iterations=..., keylen=None, hashfunc=None): ...
def safe_str_cmp(a, b): ...
def gen_salt(length): ...
def generate_password_hash(password, method='', salt_length=8): ...
def check_password_hash(pwhash, password): ...
def safe_join(directory, filename): ...



# Stubs for tarfile

from typing import (
    Callable, IO, Iterable, Iterator, List, Mapping, Optional, Type,
    Union,
)
import os
import sys
from types import TracebackType


if sys.version_info >= (3, 6):
    _Path = Union[bytes, str, os.PathLike]
elif sys.version_info >= (3,):
    _Path = Union[bytes, str]
else:
    _Path = Union[str, unicode]

ENCODING = ...  # type: str

USTAR_FORMAT = ...  # type: int
GNU_FORMAT = ...  # type: int
PAX_FORMAT = ...  # type: int
DEFAULT_FORMAT = ...  # type: int

REGTYPE = ...  # type: bytes
AREGTYPE = ...  # type: bytes
LNKTYPE = ...  # type: bytes
SYMTYPE = ...  # type: bytes
DIRTYPE = ...  # type: bytes
FIFOTYPE = ...  # type: bytes
CONTTYPE = ...  # type: bytes
CHRTYPE = ...  # type: bytes
BLKTYPE = ...  # type: bytes
GNUTYPE_SPARSE = ...  # type: bytes

if sys.version_info < (3,):
    TAR_PLAIN = ...  # type: int
    TAR_GZIPPED = ...  # type: int

def open(name: Optional[_Path] = ..., mode: str = ...,
        fileobj: Optional[IO[bytes]] = ..., bufsize: int = ...,
        *, format: Optional[int] = ..., tarinfo: Optional[TarInfo] = ...,
        dereference: Optional[bool] = ...,
        ignore_zeros: Optional[bool] = ...,
        encoding: Optional[str] = ..., errors: str = ...,
        pax_headers: Optional[Mapping[str, str]] = ...,
        debug: Optional[int] = ...,
        errorlevel: Optional[int] = ...,
        compresslevel: Optional[int] = ...) -> TarFile: ...


class TarFile(Iterable[TarInfo]):
    name = ...  # type: Optional[_Path]
    mode = ...  # type: str
    fileobj = ...  # type: Optional[IO[bytes]]
    format = ...  # type: Optional[int]
    tarinfo = ...  # type: Optional[TarInfo]
    dereference = ...  # type: Optional[bool]
    ignore_zeros = ...  # type: Optional[bool]
    encoding = ...  # type: Optional[str]
    errors = ...  # type: str
    pax_headers = ...  # type: Optional[Mapping[str, str]]
    debug = ...  # type: Optional[int]
    errorlevel = ...  # type: Optional[int]
    if sys.version_info < (3,):
        posix = ...  # type: bool
    def __init__(self, name: Optional[_Path] = ..., mode: str = ...,
                 fileobj: Optional[IO[bytes]] = ...,
                 format: Optional[int] = ..., tarinfo: Optional[TarInfo] = ...,
                 dereference: Optional[bool] = ...,
                 ignore_zeros: Optional[bool] = ...,
                 encoding: Optional[str] = ..., errors: str = ...,
                 pax_headers: Optional[Mapping[str, str]] = ...,
                 debug: Optional[int] = ...,
                 errorlevel: Optional[int] = ...,
                 compresslevel: Optional[int] = ...) -> None: ...
    def __enter__(self) -> TarFile: ...
    def __exit__(self,
                 exc_type: Optional[Type[BaseException]],
                 exc_val: Optional[Exception],
                 exc_tb: Optional[TracebackType]) -> bool: ...
    def __iter__(self) -> Iterator[TarInfo]: ...
    @classmethod
    def open(cls, name: Optional[_Path] = ..., mode: str = ...,
             fileobj: Optional[IO[bytes]] = ..., bufsize: int = ...,
             *, format: Optional[int] = ..., tarinfo: Optional[TarInfo] = ...,
             dereference: Optional[bool] = ...,
             ignore_zeros: Optional[bool] = ...,
             encoding: Optional[str] = ..., errors: str = ...,
             pax_headers: Optional[Mapping[str, str]] = ...,
             debug: Optional[int] = ...,
             errorlevel: Optional[int] = ...) -> TarFile: ...
    def getmember(self, name: str) -> TarInfo: ...
    def getmembers(self) -> List[TarInfo]: ...
    def getnames(self) -> List[str]: ...
    if sys.version_info >= (3, 5):
        def list(self, verbose: bool = ...,
                 *, members: Optional[List[TarInfo]] = ...) -> None: ...
    else:
        def list(self, verbose: bool = ...) -> None: ...
    def next(self) -> Optional[TarInfo]: ...
    if sys.version_info >= (3, 5):
        def extractall(self, path: _Path = ...,
                       members: Optional[List[TarInfo]] = ...,
                       *, numeric_owner: bool = ...) -> None: ...
    else:
        def extractall(self, path: _Path = ...,
                       members: Optional[List[TarInfo]] = ...) -> None: ...
    if sys.version_info >= (3, 5):
        def extract(self, member: Union[str, TarInfo], path: _Path = ...,
                    set_attrs: bool = ...,
                    *, numeric_owner: bool = ...) -> None: ...
    elif sys.version_info >= (3,):
        def extract(self, member: Union[str, TarInfo], path: _Path = ...,
                    set_attrs: bool = ...) -> None: ...
    else:
        def extract(self, member: Union[str, TarInfo],
                    path: _Path = ...) -> None: ...
    def extractfile(self,
                    member: Union[str, TarInfo]) -> Optional[IO[bytes]]: ...
    if sys.version_info >= (3, 7):
        def add(self, name: str, arcname: Optional[str] = ...,
                recursive: bool = ..., *,
                filter: Optional[Callable[[TarInfo], Optional[TarInfo]]] = ...) -> None: ...
    elif sys.version_info >= (3,):
        def add(self, name: str, arcname: Optional[str] = ...,
                recursive: bool = ...,
                exclude: Optional[Callable[[str], bool]] = ..., *,
                filter: Optional[Callable[[TarInfo], Optional[TarInfo]]] = ...) -> None: ...
    else:
        def add(self, name: str, arcname: Optional[str] = ...,
                recursive: bool = ...,
                exclude: Optional[Callable[[str], bool]] = ...,
                filter: Optional[Callable[[TarInfo], Optional[TarInfo]]] = ...) -> None: ...
    def addfile(self, tarinfo: TarInfo,
                fileobj: Optional[IO[bytes]] = ...) -> None: ...
    def gettarinfo(self, name: Optional[str] = ...,
                   arcname: Optional[str] = ...,
                   fileobj: Optional[IO[bytes]] = ...) -> TarInfo: ...
    def close(self) -> None: ...


def is_tarfile(name: str) -> bool: ...


if sys.version_info < (3,):
    class TarFileCompat:
        def __init__(self, filename: str, mode: str = ...,
                     compression: int = ...) -> None: ...


class TarError(Exception): ...
class ReadError(TarError): ...
class CompressionError(TarError): ...
class StreamError(TarError): ...
class ExtractError(TarError): ...
class HeaderError(TarError): ...


class TarInfo:
    name = ...  # type: str
    size = ...  # type: int
    mtime = ...  # type: int
    mode = ...  # type: int
    type = ...  # type: bytes
    linkname = ...  # type: str
    uid = ...  # type: int
    gid = ...  # type: int
    uname = ...  # type: str
    gname = ...  # type: str
    pax_headers = ...  # type: Mapping[str, str]
    def __init__(self, name: str = ...) -> None: ...
    if sys.version_info >= (3,):
        @classmethod
        def frombuf(cls, buf: bytes, encoding: str, errors: str) -> TarInfo: ...
    else:
        @classmethod
        def frombuf(cls, buf: bytes) -> TarInfo: ...
    @classmethod
    def fromtarfile(cls, tarfile: TarFile) -> TarInfo: ...
    def tobuf(self, format: Optional[int] = ...,
              encoding: Optional[str] = ..., errors: str = ...) -> bytes: ...
    def isfile(self) -> bool: ...
    def isreg(self) -> bool: ...
    def isdir(self) -> bool: ...
    def issym(self) -> bool: ...
    def islnk(self) -> bool: ...
    def ischr(self) -> bool: ...
    def isblk(self) -> bool: ...
    def isfifo(self) -> bool: ...
    def isdev(self) -> bool: ...



import sys
from asyncio.events import AbstractEventLoop
from .coroutines import coroutine
from .futures import Future
from typing import Any, Generator, Generic, List, TypeVar

__all__: List[str]


class QueueEmpty(Exception): ...
class QueueFull(Exception): ...

_T = TypeVar('_T')

class Queue(Generic[_T]):
    def __init__(self, maxsize: int = ..., *, loop: AbstractEventLoop = ...) -> None: ...
    def _init(self, maxsize: int) -> None: ...
    def _get(self) -> _T: ...
    def _put(self, item: _T) -> None: ...
    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...
    def _format(self) -> str: ...
    def _consume_done_getters(self) -> None: ...
    def _consume_done_putters(self) -> None: ...
    def qsize(self) -> int: ...
    @property
    def maxsize(self) -> int: ...
    def empty(self) -> bool: ...
    def full(self) -> bool: ...
    @coroutine
    def put(self, item: _T) -> Generator[Any, None, None]: ...
    def put_nowait(self, item: _T) -> None: ...
    @coroutine
    def get(self) -> Generator[Any, None, _T]: ...
    def get_nowait(self) -> _T: ...
    if sys.version_info >= (3, 4):
        @coroutine
        def join(self) -> Generator[Any, None, bool]: ...
        def task_done(self) -> None: ...


class PriorityQueue(Queue[_T]): ...


class LifoQueue(Queue[_T]): ...

if sys.version_info < (3, 5):
    class JoinableQueue(Queue[_T]):
        def task_done(self) -> None: ...
        @coroutine
        def join(self) -> Generator[Any, None, bool]: ...



# Stubs for email.contentmanager (Python 3.4)

from typing import Any, Callable
import sys
from email.message import Message

if sys.version_info >= (3, 4):

    class ContentManager:
        def __init__(self) -> None: ...
        def get_content(self, msg: Message, *args: Any, **kw: Any) -> Any: ...
        def set_content(self, msg: Message, obj: Any, *args: Any,
                        **kw: Any) -> Any: ...
        def add_get_handler(self, key: str, handler: Callable[..., Any]) -> None: ...
        def add_set_handler(self, typekey: type,
                            handler: Callable[..., Any]) -> None: ...

    raw_data_manager = ...  # type: ContentManager



from typing import Any

MBLENGTH = ...  # type: Any

class Charset:
    is_default = ...  # type: Any
    def __init__(self, id, name, collation, is_default): ...

class Charsets:
    def __init__(self): ...
    def add(self, c): ...
    def by_id(self, id): ...
    def by_name(self, name): ...

def charset_by_name(name): ...
def charset_by_id(id): ...



from importlib import util
from importlib.abc import Loader
import sys
import types
from typing import Any, Mapping, Optional, Sequence

def __import__(name: str, globals: Optional[Mapping[str, Any]] = ...,
               locals: Optional[Mapping[str, Any]] = ...,
               fromlist: Sequence[str] = ...,
               level: int = ...) -> types.ModuleType: ...

def import_module(name: str, package: Optional[str] = ...) -> types.ModuleType: ...

if sys.version_info >= (3, 3):
    def find_loader(name: str, path: Optional[str] = ...) -> Optional[Loader]: ...

    def invalidate_caches() -> None: ...

if sys.version_info >= (3, 4):
    def reload(module: types.ModuleType) -> types.ModuleType: ...



# Stubs for user (Python 2)

# Docs: https://docs.python.org/2/library/user.html
# Source: https://hg.python.org/cpython/file/2.7/Lib/user.py
from typing import Any

def __getattr__(name) -> Any: ...  # type: ignore
home: str
pythonrc: str



from typing import Any

log = ...  # type: Any

class DataWatch:
    def __init__(self, client, path, func=..., *args, **kwargs) -> None: ...
    def __call__(self, func): ...

class ChildrenWatch:
    def __init__(self, client, path, func=..., allow_session_lost=..., send_event=...) -> None: ...
    def __call__(self, func): ...

class PatientChildrenWatch:
    client = ...  # type: Any
    path = ...  # type: Any
    children = ...  # type: Any
    time_boundary = ...  # type: Any
    children_changed = ...  # type: Any
    def __init__(self, client, path, time_boundary=...) -> None: ...
    asy = ...  # type: Any
    def start(self): ...



from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer,
)
from google.protobuf.message import (
    Message,
)
from google.protobuf.unittest_import_pb2 import (
    ImportMessage,
)
from google.protobuf.unittest_import_public_pb2 import (
    PublicImportMessage,
)
from typing import (
    Iterable,
    List,
    Optional,
    Text,
    Tuple,
    cast,
)


class ForeignEnum(int):

    @classmethod
    def Name(cls, number: int) -> str: ...

    @classmethod
    def Value(cls, name: str) -> ForeignEnum: ...

    @classmethod
    def keys(cls) -> List[str]: ...

    @classmethod
    def values(cls) -> List[ForeignEnum]: ...

    @classmethod
    def items(cls) -> List[Tuple[str, ForeignEnum]]: ...


FOREIGN_ZERO: ForeignEnum
FOREIGN_FOO: ForeignEnum
FOREIGN_BAR: ForeignEnum
FOREIGN_BAZ: ForeignEnum


class TestAllTypes(Message):

    class NestedEnum(int):

        @classmethod
        def Name(cls, number: int) -> str: ...

        @classmethod
        def Value(cls, name: str) -> TestAllTypes.NestedEnum: ...

        @classmethod
        def keys(cls) -> List[str]: ...

        @classmethod
        def values(cls) -> List[TestAllTypes.NestedEnum]: ...

        @classmethod
        def items(cls) -> List[Tuple[str, TestAllTypes.NestedEnum]]: ...
    ZERO: NestedEnum
    FOO: NestedEnum
    BAR: NestedEnum
    BAZ: NestedEnum
    NEG: NestedEnum

    class NestedMessage(Message):
        bb = ...  # type: int

        def __init__(self,
                     bb: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(cls, s: str) -> TestAllTypes.NestedMessage: ...
    optional_int32 = ...  # type: int
    optional_int64 = ...  # type: int
    optional_uint32 = ...  # type: int
    optional_uint64 = ...  # type: int
    optional_sint32 = ...  # type: int
    optional_sint64 = ...  # type: int
    optional_fixed32 = ...  # type: int
    optional_fixed64 = ...  # type: int
    optional_sfixed32 = ...  # type: int
    optional_sfixed64 = ...  # type: int
    optional_float = ...  # type: float
    optional_double = ...  # type: float
    optional_bool = ...  # type: bool
    optional_string = ...  # type: Text
    optional_bytes = ...  # type: str
    optional_nested_enum = ...  # type: TestAllTypes.NestedEnum
    optional_foreign_enum = ...  # type: ForeignEnum
    optional_string_piece = ...  # type: Text
    optional_cord = ...  # type: Text
    repeated_int32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_int64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_uint32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_uint64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_sint32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_sint64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_fixed32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_fixed64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_sfixed32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_sfixed64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_float = ...  # type: RepeatedScalarFieldContainer[float]
    repeated_double = ...  # type: RepeatedScalarFieldContainer[float]
    repeated_bool = ...  # type: RepeatedScalarFieldContainer[bool]
    repeated_string = ...  # type: RepeatedScalarFieldContainer[Text]
    repeated_bytes = ...  # type: RepeatedScalarFieldContainer[str]
    repeated_nested_enum = ...  # type: RepeatedScalarFieldContainer[TestAllTypes.NestedEnum]
    repeated_foreign_enum = ...  # type: RepeatedScalarFieldContainer[ForeignEnum]
    repeated_string_piece = ...  # type: RepeatedScalarFieldContainer[Text]
    repeated_cord = ...  # type: RepeatedScalarFieldContainer[Text]
    oneof_uint32 = ...  # type: int
    oneof_string = ...  # type: Text
    oneof_bytes = ...  # type: str

    @property
    def optional_nested_message(self) -> TestAllTypes.NestedMessage: ...

    @property
    def optional_foreign_message(self) -> ForeignMessage: ...

    @property
    def optional_import_message(self) -> ImportMessage: ...

    @property
    def optional_public_import_message(self) -> PublicImportMessage: ...

    @property
    def optional_lazy_message(self) -> TestAllTypes.NestedMessage: ...

    @property
    def optional_lazy_import_message(self) -> ImportMessage: ...

    @property
    def repeated_nested_message(
        self) -> RepeatedCompositeFieldContainer[TestAllTypes.NestedMessage]: ...

    @property
    def repeated_foreign_message(
        self) -> RepeatedCompositeFieldContainer[ForeignMessage]: ...

    @property
    def repeated_import_message(
        self) -> RepeatedCompositeFieldContainer[ImportMessage]: ...

    @property
    def repeated_lazy_message(
        self) -> RepeatedCompositeFieldContainer[TestAllTypes.NestedMessage]: ...

    @property
    def oneof_nested_message(self) -> TestAllTypes.NestedMessage: ...

    def __init__(self,
                 optional_int32: Optional[int] = ...,
                 optional_int64: Optional[int] = ...,
                 optional_uint32: Optional[int] = ...,
                 optional_uint64: Optional[int] = ...,
                 optional_sint32: Optional[int] = ...,
                 optional_sint64: Optional[int] = ...,
                 optional_fixed32: Optional[int] = ...,
                 optional_fixed64: Optional[int] = ...,
                 optional_sfixed32: Optional[int] = ...,
                 optional_sfixed64: Optional[int] = ...,
                 optional_float: Optional[float] = ...,
                 optional_double: Optional[float] = ...,
                 optional_bool: Optional[bool] = ...,
                 optional_string: Optional[Text] = ...,
                 optional_bytes: Optional[str] = ...,
                 optional_nested_message: Optional[TestAllTypes.NestedMessage] = ...,
                 optional_foreign_message: Optional[ForeignMessage] = ...,
                 optional_import_message: Optional[ImportMessage] = ...,
                 optional_nested_enum: Optional[TestAllTypes.NestedEnum] = ...,
                 optional_foreign_enum: Optional[ForeignEnum] = ...,
                 optional_string_piece: Optional[Text] = ...,
                 optional_cord: Optional[Text] = ...,
                 optional_public_import_message: Optional[PublicImportMessage] = ...,
                 optional_lazy_message: Optional[TestAllTypes.NestedMessage] = ...,
                 optional_lazy_import_message: Optional[ImportMessage] = ...,
                 repeated_int32: Optional[Iterable[int]] = ...,
                 repeated_int64: Optional[Iterable[int]] = ...,
                 repeated_uint32: Optional[Iterable[int]] = ...,
                 repeated_uint64: Optional[Iterable[int]] = ...,
                 repeated_sint32: Optional[Iterable[int]] = ...,
                 repeated_sint64: Optional[Iterable[int]] = ...,
                 repeated_fixed32: Optional[Iterable[int]] = ...,
                 repeated_fixed64: Optional[Iterable[int]] = ...,
                 repeated_sfixed32: Optional[Iterable[int]] = ...,
                 repeated_sfixed64: Optional[Iterable[int]] = ...,
                 repeated_float: Optional[Iterable[float]] = ...,
                 repeated_double: Optional[Iterable[float]] = ...,
                 repeated_bool: Optional[Iterable[bool]] = ...,
                 repeated_string: Optional[Iterable[Text]] = ...,
                 repeated_bytes: Optional[Iterable[str]] = ...,
                 repeated_nested_message: Optional[Iterable[TestAllTypes.NestedMessage]] = ...,
                 repeated_foreign_message: Optional[Iterable[ForeignMessage]] = ...,
                 repeated_import_message: Optional[Iterable[ImportMessage]] = ...,
                 repeated_nested_enum: Optional[Iterable[TestAllTypes.NestedEnum]] = ...,
                 repeated_foreign_enum: Optional[Iterable[ForeignEnum]] = ...,
                 repeated_string_piece: Optional[Iterable[Text]] = ...,
                 repeated_cord: Optional[Iterable[Text]] = ...,
                 repeated_lazy_message: Optional[Iterable[TestAllTypes.NestedMessage]] = ...,
                 oneof_uint32: Optional[int] = ...,
                 oneof_nested_message: Optional[TestAllTypes.NestedMessage] = ...,
                 oneof_string: Optional[Text] = ...,
                 oneof_bytes: Optional[str] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> TestAllTypes: ...


class TestPackedTypes(Message):
    packed_int32 = ...  # type: RepeatedScalarFieldContainer[int]
    packed_int64 = ...  # type: RepeatedScalarFieldContainer[int]
    packed_uint32 = ...  # type: RepeatedScalarFieldContainer[int]
    packed_uint64 = ...  # type: RepeatedScalarFieldContainer[int]
    packed_sint32 = ...  # type: RepeatedScalarFieldContainer[int]
    packed_sint64 = ...  # type: RepeatedScalarFieldContainer[int]
    packed_fixed32 = ...  # type: RepeatedScalarFieldContainer[int]
    packed_fixed64 = ...  # type: RepeatedScalarFieldContainer[int]
    packed_sfixed32 = ...  # type: RepeatedScalarFieldContainer[int]
    packed_sfixed64 = ...  # type: RepeatedScalarFieldContainer[int]
    packed_float = ...  # type: RepeatedScalarFieldContainer[float]
    packed_double = ...  # type: RepeatedScalarFieldContainer[float]
    packed_bool = ...  # type: RepeatedScalarFieldContainer[bool]
    packed_enum = ...  # type: RepeatedScalarFieldContainer[ForeignEnum]

    def __init__(self,
                 packed_int32: Optional[Iterable[int]] = ...,
                 packed_int64: Optional[Iterable[int]] = ...,
                 packed_uint32: Optional[Iterable[int]] = ...,
                 packed_uint64: Optional[Iterable[int]] = ...,
                 packed_sint32: Optional[Iterable[int]] = ...,
                 packed_sint64: Optional[Iterable[int]] = ...,
                 packed_fixed32: Optional[Iterable[int]] = ...,
                 packed_fixed64: Optional[Iterable[int]] = ...,
                 packed_sfixed32: Optional[Iterable[int]] = ...,
                 packed_sfixed64: Optional[Iterable[int]] = ...,
                 packed_float: Optional[Iterable[float]] = ...,
                 packed_double: Optional[Iterable[float]] = ...,
                 packed_bool: Optional[Iterable[bool]] = ...,
                 packed_enum: Optional[Iterable[ForeignEnum]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> TestPackedTypes: ...


class TestUnpackedTypes(Message):
    repeated_int32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_int64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_uint32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_uint64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_sint32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_sint64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_fixed32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_fixed64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_sfixed32 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_sfixed64 = ...  # type: RepeatedScalarFieldContainer[int]
    repeated_float = ...  # type: RepeatedScalarFieldContainer[float]
    repeated_double = ...  # type: RepeatedScalarFieldContainer[float]
    repeated_bool = ...  # type: RepeatedScalarFieldContainer[bool]
    repeated_nested_enum = ...  # type: RepeatedScalarFieldContainer[TestAllTypes.NestedEnum]

    def __init__(self,
                 repeated_int32: Optional[Iterable[int]] = ...,
                 repeated_int64: Optional[Iterable[int]] = ...,
                 repeated_uint32: Optional[Iterable[int]] = ...,
                 repeated_uint64: Optional[Iterable[int]] = ...,
                 repeated_sint32: Optional[Iterable[int]] = ...,
                 repeated_sint64: Optional[Iterable[int]] = ...,
                 repeated_fixed32: Optional[Iterable[int]] = ...,
                 repeated_fixed64: Optional[Iterable[int]] = ...,
                 repeated_sfixed32: Optional[Iterable[int]] = ...,
                 repeated_sfixed64: Optional[Iterable[int]] = ...,
                 repeated_float: Optional[Iterable[float]] = ...,
                 repeated_double: Optional[Iterable[float]] = ...,
                 repeated_bool: Optional[Iterable[bool]] = ...,
                 repeated_nested_enum: Optional[Iterable[TestAllTypes.NestedEnum]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> TestUnpackedTypes: ...


class NestedTestAllTypes(Message):

    @property
    def child(self) -> NestedTestAllTypes: ...

    @property
    def payload(self) -> TestAllTypes: ...

    @property
    def repeated_child(
        self) -> RepeatedCompositeFieldContainer[NestedTestAllTypes]: ...

    def __init__(self,
                 child: Optional[NestedTestAllTypes] = ...,
                 payload: Optional[TestAllTypes] = ...,
                 repeated_child: Optional[Iterable[NestedTestAllTypes]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> NestedTestAllTypes: ...


class ForeignMessage(Message):
    c = ...  # type: int

    def __init__(self,
                 c: Optional[int] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> ForeignMessage: ...


class TestEmptyMessage(Message):

    def __init__(self,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: str) -> TestEmptyMessage: ...



# Stubs for email.parser (Python 3.4)

import email.feedparser
from email.message import Message
import sys
from typing import Callable, Optional, TextIO, BinaryIO
if sys.version_info >= (3, 3):
    from email.policy import Policy

FeedParser = email.feedparser.FeedParser
BytesFeedParser = email.feedparser.BytesFeedParser

class Parser:
    if sys.version_info >= (3, 3):
        def __init__(self, _class: Callable[[], Message] = ..., *,
                     policy: Policy = ...) -> None: ...
    else:
        # TODO `strict` is positional
        def __init__(self,
                     _class: Callable[[], Message] = ..., *,
                     strict: Optional[bool]) -> None: ...
    def parse(self, fp: TextIO, headersonly: bool = ...) -> Message: ...
    def parsestr(self, text: str, headersonly: bool = ...) -> Message: ...

class HeaderParser(Parser):
    if sys.version_info >= (3, 3):
        def __init__(self, _class: Callable[[], Message] = ..., *,
                     policy: Policy = ...) -> None: ...
    else:
        # TODO `strict` is positional
        def __init__(self,
                     _class: Callable[[], Message] = ..., *,
                     strict: Optional[bool]) -> None: ...
    def parse(self, fp: TextIO, headersonly: bool = ...) -> Message: ...
    def parsestr(self, text: str, headersonly: bool = ...) -> Message: ...

if sys.version_info >= (3, 3):
    class BytesHeaderParser(BytesParser):
        if sys.version_info >= (3, 3):
            def __init__(self, _class: Callable[[], Message] = ..., *,
                         policy: Policy = ...) -> None: ...
        else:
            # TODO `strict` is positional
            def __init__(self,
                         _class: Callable[[], Message] = ..., *,
                         strict: Optional[bool]) -> None: ...
        def parse(self, fp: BinaryIO, headersonly: bool = ...) -> Message: ...
        def parsestr(self, text: str, headersonly: bool = ...) -> Message: ...

if sys.version_info >= (3, 2):
    class BytesParser:
        if sys.version_info >= (3, 3):
            def __init__(self, _class: Callable[[], Message] = ..., *,
                         policy: Policy = ...) -> None: ...
        else:
            # TODO `strict` is positional
            def __init__(self,
                         _class: Callable[[], Message] = ..., *,
                         strict: Optional[bool]) -> None: ...
        def parse(self, fp: BinaryIO, headersonly: bool = ...) -> Message: ...
        def parsestr(self, text: str, headersonly: bool = ...) -> Message: ...



from distutils.version import Version
from typing import Any, Callable, Dict, List, Optional, Type, TypeVar, Union, Text

from click.core import Command, Group, Argument, Option, Parameter, Context
from click.types import _ConvertibleType

_T = TypeVar('_T')
_Decorator = Callable[[_T], _T]

_Callback = Callable[
    [Context, Union[Option, Parameter], Union[bool, int, str]],
    Any
]

def pass_context(_T) -> _T:
    ...


def pass_obj(_T) -> _T:
    ...


def make_pass_decorator(
    object_type: type, ensure: bool = ...
) -> Callable[[_T], _T]:
    ...


# NOTE: Decorators below have **attrs converted to concrete constructor
# arguments from core.pyi to help with type checking.

def command(
    name: Optional[str] = ...,
    cls: Optional[Type[Command]] = ...,
    # Command
    context_settings: Optional[Dict] = ...,
    help: Optional[str] = ...,
    epilog: Optional[str] = ...,
    short_help: Optional[str] = ...,
    options_metavar: str = ...,
    add_help_option: bool = ...,
) -> _Decorator:
    ...


# This inherits attrs from Group, MultiCommand and Command.

def group(
    name: Optional[str] = ...,
    cls: Type[Command] = ...,
    # Group
    commands: Optional[Dict[str, Command]] = ...,
    # MultiCommand
    invoke_without_command: bool = ...,
    no_args_is_help: Optional[bool] = ...,
    subcommand_metavar: Optional[str] = ...,
    chain: bool = ...,
    result_callback: Optional[Callable] = ...,
    # Command
    help: Optional[str] = ...,
    epilog: Optional[str] = ...,
    short_help: Optional[str] = ...,
    options_metavar: str = ...,
    add_help_option: bool = ...,
    # User-defined
    **kwargs: Any,
) -> _Decorator:
    ...


def argument(
    *param_decls: str,
    cls: Type[Argument] = ...,
    # Argument
    required: Optional[bool] = ...,
    # Parameter
    type: Optional[_ConvertibleType] = ...,
    default: Optional[Any] = ...,
    callback: Optional[_Callback] = ...,
    nargs: Optional[int] = ...,
    metavar: Optional[str] = ...,
    expose_value: bool = ...,
    is_eager: bool = ...,
    envvar: Optional[Union[str, List[str]]] = ...
) -> _Decorator:
    ...


def option(
    *param_decls: str,
    cls: Type[Option] = ...,
    # Option
    show_default: bool = ...,
    prompt: Union[bool, Text] = ...,
    confirmation_prompt: bool = ...,
    hide_input: bool = ...,
    is_flag: Optional[bool] = ...,
    flag_value: Optional[Any] = ...,
    multiple: bool = ...,
    count: bool = ...,
    allow_from_autoenv: bool = ...,
    type: Optional[_ConvertibleType] = ...,
    help: Optional[str] = ...,
    # Parameter
    default: Optional[Any] = ...,
    required: bool = ...,
    callback: Optional[_Callback] = ...,
    nargs: Optional[int] = ...,
    metavar: Optional[str] = ...,
    expose_value: bool = ...,
    is_eager: bool = ...,
    envvar: Optional[Union[str, List[str]]] = ...,
    # User-defined
    **kwargs: Any,
) -> _Decorator:
    ...


def confirmation_option(
    *param_decls: str,
    cls: Type[Option] = ...,
    # Option
    show_default: bool = ...,
    prompt: Union[bool, Text] = ...,
    confirmation_prompt: bool = ...,
    hide_input: bool = ...,
    is_flag: bool = ...,
    flag_value: Optional[Any] = ...,
    multiple: bool = ...,
    count: bool = ...,
    allow_from_autoenv: bool = ...,
    type: Optional[_ConvertibleType] = ...,
    help: str = ...,
    # Parameter
    default: Optional[Any] = ...,
    callback: Optional[_Callback] = ...,
    nargs: Optional[int] = ...,
    metavar: Optional[str] = ...,
    expose_value: bool = ...,
    is_eager: bool = ...,
    envvar: Optional[Union[str, List[str]]] = ...
) -> _Decorator:
    ...


def password_option(
    *param_decls: str,
    cls: Type[Option] = ...,
    # Option
    show_default: bool = ...,
    prompt: Union[bool, Text] = ...,
    confirmation_prompt: bool = ...,
    hide_input: bool = ...,
    is_flag: Optional[bool] = ...,
    flag_value: Optional[Any] = ...,
    multiple: bool = ...,
    count: bool = ...,
    allow_from_autoenv: bool = ...,
    type: Optional[_ConvertibleType] = ...,
    help: Optional[str] = ...,
    # Parameter
    default: Optional[Any] = ...,
    callback: Optional[_Callback] = ...,
    nargs: Optional[int] = ...,
    metavar: Optional[str] = ...,
    expose_value: bool = ...,
    is_eager: bool = ...,
    envvar: Optional[Union[str, List[str]]] = ...
) -> _Decorator:
    ...


def version_option(
    version: Optional[Union[str, Version]] = ...,
    *param_decls: str,
    cls: Type[Option] = ...,
    # Option
    prog_name: Optional[str] = ...,
    message: Optional[str] = ...,
    show_default: bool = ...,
    prompt: Union[bool, Text] = ...,
    confirmation_prompt: bool = ...,
    hide_input: bool = ...,
    is_flag: bool = ...,
    flag_value: Optional[Any] = ...,
    multiple: bool = ...,
    count: bool = ...,
    allow_from_autoenv: bool = ...,
    type: Optional[_ConvertibleType] = ...,
    help: str = ...,
    # Parameter
    default: Optional[Any] = ...,
    callback: Optional[_Callback] = ...,
    nargs: Optional[int] = ...,
    metavar: Optional[str] = ...,
    expose_value: bool = ...,
    is_eager: bool = ...,
    envvar: Optional[Union[str, List[str]]] = ...
) -> _Decorator:
    ...


def help_option(
    *param_decls: str,
    cls: Type[Option] = ...,
    # Option
    show_default: bool = ...,
    prompt: Union[bool, Text] = ...,
    confirmation_prompt: bool = ...,
    hide_input: bool = ...,
    is_flag: bool = ...,
    flag_value: Optional[Any] = ...,
    multiple: bool = ...,
    count: bool = ...,
    allow_from_autoenv: bool = ...,
    type: Optional[_ConvertibleType] = ...,
    help: str = ...,
    # Parameter
    default: Optional[Any] = ...,
    callback: Optional[_Callback] = ...,
    nargs: Optional[int] = ...,
    metavar: Optional[str] = ...,
    expose_value: bool = ...,
    is_eager: bool = ...,
    envvar: Optional[Union[str, List[str]]] = ...
) -> _Decorator:
    ...
